/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								            **
***                    COPYRIGHT 2011 INTEL CORPORATION                       **
***							All Rights Reserved								            **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		**
***		related to the source code ("Material") are owned by Intel			   **
***		Corporation or its suppliers or licensors. Title to the Material	   **
***		remains with Intel Corporation or its suppliers and licensors.		   **
***		The Material contains trade secrets and proprietary and confidential **
***		information of Intel or its suppliers and licensors.				      **
***		The Material is protected by worldwide copyright and trade secret	   **
***		laws and treaty provisions. No part of the Material may be used,	   **
***		copied, reproduced, modified, published, uploaded, posted,			   **
***		transmitted, distributed, or disclosed in any way without Intel's	   **
***		prior express written permission.									         **
***																			                  **
***		No license under any patent, copyright, trade secret or other		   **
***		intellectual property right is granted to or conferred upon you by	**
***		disclosure or delivery of the Materials, either expressly, by		   **
***		implication, inducement, estoppel or otherwise. Any license under	   **
***		such intellectual property rights must be express and approved by	   **
***		Intel in writing.													               **
********************************************************************************/
/*
 * CommonClasses.hpp
 *
 *  Created on: 21-02-2011
 */

#ifndef _COMMON_CLASSES_HPP_
#define _COMMON_CLASSES_HPP_

#include "transport.h"
#include <vector>
#include <ostream>
#include <string>
#include <iterator>
#include <algorithm>
#include <stdexcept>

/*
 template<typename T, size_t N>
 size_t copy(std::vector<T> const &src, T[N] dest)
 {
 size_t count = std::min(N, src.size());
 std::copy(src.begin(), src.begin()+count, dest);

 return count;
 }


inline ios_base& hex_showbase(ios_base& _Iosbase)
{	// set basefield to hex
	_Iosbase.setf(ios_base::hex, ios_base::basefield);
	// set showbase
	_Iosbase.setf(ios_base::showbase);
	return (_Iosbase);
}
*/


namespace CommonClasses
{
/**
 * This is base utility class.
 */
template<typename T>
class BaseClass: protected T
{

  friend class Request;
  friend class IpmiResponse;

public:
	/**
	 * Default ctor
	 *
	 * @param rhs - type
	 * @return none
	 */
	BaseClass()
	{
		//Initialize entire structure to 0
		clear();
	}

	/**
	 * Default copy ctor
	 *
	 * @param rhs - type
	 * @return none
	 */
	BaseClass(T &rhs)
	{
		operator=(rhs);
	}

	/**
	 * Default copy ctor
	 *
	 * @param rhs - type
	 * @return none
	 */
	BaseClass(BaseClass &rhs)
	{
		operator=(rhs);
	}

	/**
	 * Dtor, Nothing special to do
	 *
	 * @return none
	 */
	virtual ~BaseClass()
	{}

	// ------ operators ----------------------
	inline T* operator &()
	{
		return (static_cast<T*> (this));
	}

	BaseClass& operator=(BaseClass &rhs)
	{
		if (this != &rhs)
			*static_cast<T*>(this) = rhs;

		return (*this);
	}

	virtual void clear()
	{
		std::fill(reinterpret_cast<char*>(operator &()), reinterpret_cast<char*>(operator &()) + sizeof(T), 0);
	}
}; //BaseClass


/**
 * Abstract Class for Physical Response
 * for Response we are using decorator pattern to validate obtained data...
 */
class Response
{

public:

	Response() {};

	virtual ~Response() {};

	virtual byte_t getCompCode() = 0;
	virtual size_t getLength() = 0;
	virtual byte_t &at(size_t ) = 0;
    virtual void clear() = 0;

	virtual dcmi_rsp_t* operator &() = 0;

	/**
	 * Validation methods
	 * @return
	 */
	virtual bool isValidSize() = 0;
	virtual bool isCompletionCodeValid() = 0;
	virtual bool isValidResponse() = 0;

	virtual void print(std::ostream &stream);

	/**
	 * Print info
	 *
	 * @param stream - stream to print out data
	 * @param ob - object to print out
	 *
	 * @return stream...
	 */
	friend std::ostream &operator<<(std::ostream &stream, Response &response)
	{
		response.print(stream);
		return stream;
	}

	static std::string getCompletionCodeDescryption(byte_t compCode);

protected:

	void print_header(std::ostream &stream);

};

/**
 * Abstract Class for Physical Request
 */
class Request : public BaseClass<dcmi_req_t>
{

public:

	Request() :
		BaseClass<dcmi_req_t>()
	{};

	Request(Request &rhs) :
		BaseClass<dcmi_req_t>(rhs)
	{
	};

	Request& operator=(Request &rhs)
	{
		BaseClass<dcmi_req_t>::operator=(rhs);
		return (*this);
	}

	/**
	 * Clears the appended data ...
	 * truly in derived classes use to sets data size length to it initial values.
	 */
	virtual void clear()
	{
		BaseClass<dcmi_req_t>::clear();
		len = 0;
	}

	virtual void print (std::ostream &stream);
	/**
	 * Print info
	 *
	 * @param stream - stream to print out data
	 * @param ob - object to print out
	 *
	 * @return stream...
	 */
	friend std::ostream &operator<<(std::ostream &stream, Request &request)
	{
		request.print(stream);
		return stream;
	}
};


/**
 * This intermediate class, useful to initiate request with the defaults.
 *
 */
class IpmiRequest: public Request
{

public:

	/**
	 * default Ctor
	 * @param _netFun - net function
	 * @param _cmd - command code
	 * @param _param - additional data to send
	 */
	IpmiRequest(byte_t _netFun, byte_t _cmd) :
		//call default ctor
		Request()
	{ //Initializing individual elements
		netFun = _netFun;
		cmd = _cmd;
	}

	/**
	 *  Ctor.
	 *
	 * @param _netFun - net function
	 * @param _cmd	 - command code
	 * @param _data	 - additional data to send
	 * @return none ctor
	 */
	IpmiRequest(byte_t _netFun, byte_t _cmd, const std::vector<byte_t>& _data) :
		//call default ctor
		Request()
	{ //Initializing individual elements
		netFun = _netFun;
		cmd = _cmd;
		append(_data);
	}

	IpmiRequest(IpmiRequest &rhs) :
		Request(rhs)
	{
	};


	/**
	 *  Ctor.
	 *
	 *shuto   @param _netFun 	- net function
	 * @param _cmd	 	- command code
	 * @param _param	- additional data to send
	 * @param _len 		- additional data length
	 * @return none ctor
	 */
	IpmiRequest(byte_t _netFun, byte_t _cmd, const byte_t _param[], int _len) :
		//call default ctor
		Request()
	{ //Initializing individual elements
		netFun = _netFun;
		cmd = _cmd;
		append(_param, _len);
	}

	/**
	 * Ctor
	 * @param _netFun - net function
	 * @param _cmd - command code
	 * @param _param - additional data to send
	 */
	IpmiRequest(byte_t _netFun, byte_t _cmd, const byte_t _param) :
		//call default ctor
		Request()
	{
		//Initializing individual elements
		netFun = _netFun;
		cmd = _cmd;
		//add param to ipmi data request
		append<byte_t> (_param);
	}


	void setRsAddr(byte_t _rsAddr)
	{
		rsAddr = _rsAddr;
	}

	void setRsLun(byte_t _rsLun)
	{
		rsLun = _rsLun;
	}

	/**
	 * Appends data to request data field
	 * @param _param -  value to add
	 *
	 * @return request reference
	 */
	template<class T>
	IpmiRequest &append(const T _param)
	{
		return append (reinterpret_cast<const byte_t*>(&_param), sizeof(T));
	}

	/**
	 * Appends data to request data field
	 * @param _param - byte array to store
	 * @param _len	 - array length
	 *
	 * @return request reference ...
	 */
	IpmiRequest &append(const byte_t *_param, size_t _len)
	{
		if ((_len + len) > sizeof(data)) /*MAX_IPMI_SIZE*/
			throw std::out_of_range("Data request length too long");

		//add param data to ipmi data request
		if(_len == 1) {
			data[len++] = *_param;
		} else
			if (_len)
			{
#ifdef _MSC_VER
				std::copy(_param, _param + _len,
						stdext::checked_array_iterator<byte_t*>(&data[len], sizeof(data)-len));
#else
				std::copy(_param, _param + _len, &data[len]);
#endif
				len += _len;
			}
		return *(this);
	}

	/**
	 * Appends data to request data field
	 * @param _param - byte array
	 *
	 * @return request reference
	 */
	IpmiRequest &append(const std::vector<byte_t>& _param)
	{
		append(&_param.front(), _param.size());
		return *(this);
	}

	/**
	 * Replaces data in request data field
	 * @param _pos - value position to replace
	 * @param _param - value
	 *
	 * @return request reference
	 */
	IpmiRequest &replace(const size_t _pos, const byte_t _param)
	{
		if (_pos >= len || _pos > sizeof(data))
			throw std::out_of_range("Position out off range");
		//replace param data in ipmi data request
		data[_pos] = _param;
		return *(this);
	}

	/**
	 * Sets data length to new value
	 * @param _len
	 */
	inline void clear(size_t _len = 0)
	{
		len = _len;
	}

	byte_t &operator[] (size_t _offset)
	{
		if (len < _offset)
			throw std::out_of_range("Position out off range");
		return data[_offset];
	}

}; // IpmiRequest


/**
 *  This intermediate class, useful to initiate response .
 */
class IpmiResponse : public Response
{

    BaseClass<dcmi_rsp_t> m_respdata;

    std::vector<byte_t> m_ccodes;

	size_t m_minsize;
	size_t m_maxsize;

public:

	IpmiResponse()
	{
		m_minsize = 0;
		m_maxsize = 0;

		m_ccodes.push_back(0x00);
	}

   /**
	 *
	 * @param min_size
	 * @param max_size
	 *
	 * @param compcode first valid CCode,
	 * @param ...  list of valid comp codes - 0 is the sign terminator
	 */
	IpmiResponse(size_t min_size, size_t max_size, byte_t ccode = 0x00)
	{
		initSize(min_size,max_size);
		m_ccodes.push_back(ccode);
	}

	/**
	 *
	 * @param min_size
	 * @param max_size
	 *
	 * @param compcode first valid CCode,
	 * @param ...  list of valid comp codes - 0 is the sign terminator
	 */
	IpmiResponse(size_t min_size, size_t max_size, const byte_t ccode[], int cclen)
	{
		initSize(min_size,max_size);
		m_ccodes.assign(&ccode[0], &ccode[cclen]);
	}


	bool isValidSize()
	{
		if(!(m_minsize && m_maxsize)) //zeros skip test
			return true;
		if((m_minsize <= getLength()) && (getLength() <= m_maxsize))
			return true;

		return false;
	}

	/**
	 * The completion code returned in response to a previous request.
	 * The completion codes are as defined in IPMI v2.0 specification,
	 * section 5.2 and 5.3.
	 */
	inline byte_t getCompCode()
	{
		return m_respdata.compCode;
	}

	/**
	 * Checks if response completion code is valid ...
	 *
	 * @return true if ok/ false otherwise
	 */
	bool isCompletionCodeValid()
	{
		std::vector<byte_t>::iterator it = std::find(m_ccodes.begin(), m_ccodes.end(), getCompCode());
		return (it != m_ccodes.end());
	}

	/**
	 * Checks if response is valid ...
	 * Completion code, size and for dcmi responses can validate returned dcmi extension id
	 * @return
	 */
	bool isValidResponse()
	{
		return (isCompletionCodeValid() && isValidSize());
	}

	size_t getLength()
	{
		return m_respdata.len;
	}

	void clear()
	{
		m_respdata.clear();
	}

	byte_t &at(size_t offset)
	{
		if (getLength() < offset)
			EX_OutOfRange();

		return m_respdata.data[offset];
	}

	inline dcmi_rsp_t *operator &()
	{
		return &m_respdata;
	}

	byte_t &operator[] (size_t _offset)
	{
		return at(_offset);
	}

	template<class T>
	const T &getFromData(size_t _offset) /*throw(...)*/
			//C++ function template for taking value from spec offset of data
	{
		if ((_offset + sizeof(T)) > getLength())
			EX_OutOfRange();

		return *reinterpret_cast<T *> (&at(_offset));;
	}

	void copy_to(size_t _offset, byte_t *_dest, size_t _size);

	void copy_to(size_t _offset, std::vector<byte_t> &_dest);

protected:

	void initSize(size_t min_size, size_t max_size)
   {
	   if(/*min_size >= 0 &&*/ min_size <= max_size)
	   {
		   m_minsize = min_size;
		   m_maxsize = max_size;
	   }
   }

	inline void EX_OutOfRange()
	{
		throw std::out_of_range("Offset out of range");
	}


}; //end IpmiResponse


} //namespace CommonClasses

#endif /* _COMMONCLASSES_HPP_ */
