/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intel�s	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*******************************************************************************
***		 File		:	DCMI_Chassis_lib.h  								  **
***																			  **
********************************************************************************/

#ifndef DCMI_CHASSIS_LIB_H
#define DCMI_CHASSIS_LIB_H

//Define the Response length expected as in IPMI/DCMI spec
#define CHASSIS_CTRL_MIN_RSPLEN		00	//Min Response Length for Chassis Control Command
#define CHASSIS_CTRL_MAX_RSPLEN		00	//Max Response Length for Chassis Control Command

#define CHASSIS_STATUS_MIN_RSPLEN	03	//Min Response Length for Chassis Status Command
#define CHASSIS_STATUS_MAX_RSPLEN	04	//Max Response Length for Chassis Status Command

#define CHASSIS_IDENTIFY_MIN_RSPLEN	00	//Min Response Length for Chassis Identify Command
#define CHASSIS_IDENTIFY_MAX_RSPLEN	00	//Max Response Length for Chassis Identify Command

#define GET_ACPI_PWR_MIN_RSPLEN	2	//Min Response length for Get ACPI Power State Cmd
#define GET_ACPI_PWR_MAX_RSPLEN	2	//Max Response length for Get ACPI Power State Cmd

#define GET_CHASSIS_CAPAB_MIN_RSPLEN 06 //Min Response Length for Get Chassis Capabilities Cmd
#define GET_CHASSIS_CAPAB_MAX_RSPLEN 06 //Max Response Length for Get Chassis Capabilities Cmd

#include "DCMIClasses.hpp"

class ChassisBase : public DCMIClasses::DCMIBaseTest
{

public:

	void doConfigure(DcmiConfiguration &option) ;

protected:

	int m_TimeOutPwrOn;
	int m_TimeOutPwrOff;
	int m_TimeOutLedBlink;

	enum POWER_CMD
	{
		DC_OFF = 0,
		DC_ON = 1,
		SYSTEM_RESET = 2
	};

	ChassisBase(std::string name, TransportPlugin::TransportPlugIn &transport) :
		DCMIBaseTest(name, transport)
	{
		m_TimeOutPwrOn = 10;
		m_TimeOutLedBlink = 10;
		m_TimeOutPwrOff = 10;
	}

	static bool isACPI_OFF_State(int iSystemPwrState);
	static bool isACPI_ON_State(int iSystemPwrState);

	template<class Predicate>
	bool checkSystemState( Predicate pred );

	byte_t getAcpiPwrState ();
	byte_t getChassisStatus();

	void chassisIdentify(byte_t IdentifyInterval, byte_t ForceIdentify);
	void issueChassisControl(POWER_CMD chAction);
};


class ChassisTest: public ChassisBase
{

public:

	ChassisTest(TransportPlugin::TransportPlugIn &transport) :
		ChassisBase("Chassis Commands", transport)
	{
		ADD_TEST(ChassisTest::ChassisCapabilitiesTest, "Issue Get Chassis Capabilities Command", TestUtil::TT_MANDATORY);
		ADD_TEST(ChassisTest::ChkStatusPwrONTest, "Checking Chassis Status for Initial Power State", TestUtil::TT_MANDATORY);
		ADD_TEST(ChassisTest::ChassisIdentifyCheckTest, "Checking Chassis Identify Command supported", TestUtil::TT_MANDATORY);
		ADD_TEST(ChassisTest::AcpiPwrStateTest, "Check ACPI Power State", TestUtil::TT_MANDATORY);

		if(isRemoteConnection())
		{
			ADD_TEST(ChassisTest::ChassisTurnOffTest, "Check Turn System Off", TestUtil::TT_MANDATORY);
			ADD_TEST(ChassisTest::ChassisTurnOnTest, "Check Turn System On", TestUtil::TT_MANDATORY);
			ADD_TEST(ChassisTest::ChassisRebootTest, "Check Reboot System", TestUtil::TT_MANDATORY);
		}
	}

	TestResult ChassisCapabilitiesTest(any_value &);
	TestResult ChkStatusPwrONTest(any_value &);
	TestResult ChassisIdentifyCheckTest(any_value &);
	TestResult AcpiPwrStateTest(any_value &);

	TestResult ChassisTurnOffTest(any_value &);
	TestResult ChassisTurnOnTest(any_value &);
	TestResult ChassisRebootTest(any_value &);
};

#endif DCMI_CHASSIS_LIB_H
