/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intel�s	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*******************************************************************************
***		 File		:	DCMI_Discovery_lib.h  								  **
***																			  **
********************************************************************************/


//Define the Response length expected as in IPMI/DCMI spec
#define DCMI_CAPAB_INFO_MIN_RSPLEN	3		//Minimum expected response length for Get DCMI Capab Info Command
#define DCMI_CAPAB_INFO_MAX_RSPLEN	256		//Maximum expected response length for Get DCMI Capab Info Command

#define IPMI_DCMI_1_1_CONFORM       0x0101	//Only DCMI 1.1 spec is supported



#define GET_DEV_GUID_MIN_RSPLEN  16 //Minimum response length for GetDevice GUID command
#define GET_DEV_GUID_MAX_RSPLEN  16 //Maximum response length for GetDevice GUID command

#define GET_ASSET_MIN_RSPLEN  02    //Minimum expected response length for Get Asset Tag Command
#define GET_ASSET_MAX_RSPLEN  18    //Maximum expected response length for Get Asset Tag Command

#define  ASSET_TAG_MAX_LEN    64    //Maximum length of characters in AssetTag is 64
                              // Table 6-4 FRU product area of DCMI 1.0 spec
                              // defines this.

#define GET_ASSET_TAG_MAX_BYTES_TO_READ 16  //GET ASSET TAG command allows reading
                                 //Asset Tag 16 bytes at a time
                                 //Sec.6.4.1.1 of DCMI 1.0 spec


#include "DCMIClasses.hpp"
/**
 * Basic dcmi discovery class, used as base class for test suit
 *
 */
class DCMIDiscovery: public DCMIClasses::DCMIBaseTest
{
	int m_Ver;

	//Struts will be modified during the tests executing
	//for Parameter 1 - Supported DCMI Capabilities
	s_supported_DCMI_capab s_DCMICapab;
	//for Parameter 2 - Minimum Platform attributes
	s_Min_Platform_Attrib s_MinAttrib;
	//for Parameter 3 - Optional Platform Attributes
	s_Opt_Platform_Attrib s_OptAttrib;
	//for Parameter 4 - Manageability Access Attributes
	s_Mgmt_Access_Attrib s_MgmtAttrib ;
	 //for Parameter 5 - Enhanced System Power Statistics Attributes
	s_System_Power_Attrib s_SysPowerAttrib;


public:

	DCMIDiscovery(int ver, TransportPlugin::TransportPlugIn &transport);

	void doConfigure(DcmiConfiguration &option);

//dummy test
	TestResult PlatformSuppCapabTest(any_value &)
	{
		return TestUtil::TR_SUCCESS;
	}
//dummy test
	TestResult MgmtAccessAttribTest(any_value &)
	{
		return TestUtil::TR_SUCCESS;
	}

	TestResult PlatformSuppCapabSessionLessTest(any_value &);

	TestResult MinPlatformAttribTest(any_value &)
	{
		return MinPlatformAttrib(s_MinAttrib);
	}

	TestResult OptPlatformAttribTest(any_value &)
	{
		if (getCapbilities().sopt_capab.bPowerMgmt == 1 )
			return OptPlatformAttrib(s_OptAttrib);

		return TestUtil::TR_SKIPPED;
	}

	TestResult SystemPowerAttribTest(any_value &)
	{
		if (s_DCMICapab.sopt_capab.bPowerMgmt == 1 )
			return SystemPowerAttrib(getCapbilities(), s_SysPowerAttrib);

		return TestUtil::TR_SKIPPED;
	}

	 //Display the Capability Summary in a Readable Format - Supported / NOT Supported.
	TestResult CapabilitySumaryInfo(TestUtil::TestOutput &);


protected:

	inline s_supported_DCMI_capab &getCapbilities() /*throw(...)*/
	{
		return s_DCMICapab;
	}

	inline s_Min_Platform_Attrib &getMinPlatformAttrib() /*throw(...)*/
	{
		return s_MinAttrib;
	}

	inline s_Opt_Platform_Attrib &getOptPlatformAttrib() /*throw(...)*/
	{
		return s_OptAttrib;
	}

	inline s_Mgmt_Access_Attrib &getMgmtAccessAttrib() /*throw(...)*/
	{
		return 	s_MgmtAttrib;
	}

	TestResult
	PlatformSuppCapab(s_supported_DCMI_capab &s_DCMICapab);

	TestResult
	MinPlatformAttrib(s_Min_Platform_Attrib &s_MinAttrib);

	TestResult
	OptPlatformAttrib(s_Opt_Platform_Attrib &s_OptAttrib);

	TestResult
	MgmtAccessAttrib(s_supported_DCMI_capab &s_DCMICapab, s_Mgmt_Access_Attrib &s_MgmtAttrib);

	TestResult
	SystemPowerAttrib(s_supported_DCMI_capab &s_DCMICapab, s_System_Power_Attrib &s_SysPowerAttrib);

	void getDCMICapabInfoCmd(byte_t iParamSel, DCMIClasses::IpmiDCMIResponse &response);

	template <class T>
	T getDCMICapabInfoCmd(byte_t param);

};


template <class T>
T DCMIDiscovery::getDCMICapabInfoCmd(byte_t param)
{
	DCMIClasses::IpmiDCMIResponse response(DCMI_CAPAB_INFO_MIN_RSPLEN, DCMI_CAPAB_INFO_MAX_RSPLEN);
	getDCMICapabInfoCmd(param, response);//Get Supported DCMI Capabilities
	return response.getFromData<T>(4);
}


class DCMIBasics : public DCMIClasses::DCMIBaseTest
{

public:

	DCMIBasics(TransportPlugin::TransportPlugIn &transport):
		DCMIBaseTest("Basics Test", transport)
	{
		ADD_TEST(DCMIBasics::DeviceIdTest, "Device ID from Management Controller", TestUtil::TT_MANDATORY);
		ADD_TEST(DCMIBasics::GUIDTest, "System GUID from Management Controller", TestUtil::TT_MANDATORY);
		ADD_TEST(DCMIBasics::IssueAssetTagTest, "Asset Tag from UUT", TestUtil::TT_MANDATORY);
	}

	TestResult GUIDTest(any_value &);
	TestResult IssueAssetTagTest(any_value &);

protected:

	void getAssetTag(byte_t Offset, byte_t NumBytes, std::vector<byte_t> &resp_data);
};
