/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*******************************************************************************
***		 File		:	DCMI_Misc_lib.h		 								  **
***																			  **
********************************************************************************/

#define GET_CHANNEL_CIPHER_MIN_RSPLEN	1  //Min Response length for Get Channel CipherSuites command
#define GET_CHANNEL_CIPHER_MAX_RSPLEN	17 //Max Response length for Get Channel CipherSuites command

#include "DCMIClasses.hpp"

//Maximum CipherSuites the platform can support as per IPMI currently this could be FF
#define MAX_CIPHER_SUPP 	0xFF

using TestUtil::TestResult;

class CipherTest: public DCMIClasses::DCMIBaseTest
{
	byte_t m_channelNum;
	std::vector<byte_t> m_SpecCiphers;

public:
	CipherTest(byte_t channel, TransportPlugin::TransportPlugIn &transport) :
		DCMIBaseTest("Cipher Tests", transport)
	{
		m_channelNum = channel;

		ADD_TEST(CipherTest::PlatformCipherSuitesTest, "Checking Supported CipherSuites", TestUtil::TT_MANDATORY);
	}

	void doConfigure(DcmiConfiguration &option);

	TestResult PlatformCipherSuitesTest(any_value &);

protected:
	/**
	 * Returns CipherSuites supported in corresponding Specification current table-6 of DCMI 1.1
	 * Vitual to have possibilities to change it by inheritance
	 *
	 * @return array of supported ciphers
	 */

	virtual std::vector<byte_t> &specificationCiphers();

	/**
	 *  Verify if ALL the Ciphers in the spec are in the ciphers array
	 *  For each CipherSuite in the list of DCMI supported CipherSuites,
	 *  check if corresponding entry exists in spec array
	 * @param cipher - platform cipher list
	 * @return final cipher list compared against specification list
	 */
	std::vector<byte_t> intersectCiphers(std::vector<byte_t> &cipher);


	void getChannelCipherSuites(byte_t iPayloadType, byte_t iListIndex, std::vector<byte_t> &resp_data);

	void getPlatformCipherSuites(std::vector<byte_t> &cipher);

	/**
	 * Basic verification if cipher list is equal to specification
	 *
	 * @param cipher - array of cipher to validate
	 * @return success or fail
	 */
	virtual TestResult validateCipherSuites(std::vector<byte_t> &ciphers);

};


class CheckCiphersTest : public CipherTest
{

public:
	CheckCiphersTest(byte_t channel, TransportPlugin::TransportPlugIn &transport):
		CipherTest(channel, transport)
	{ }


	TestResult ChiperTest(any_value &cipherid);

protected:
	/**
	 * Test each Cipher in a Loop
	 * @param ciphers - cipher list to try ..
	 * @return
	 */
	virtual TestResult validateCipherSuites(std::vector<byte_t> &ciphers);

};
