/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*******************************************************************************
***		 File		:	DCMI_PwrMgmt_lib.h		  						      **
***																			  **
********************************************************************************/

//Define the Response length expected as in IPMI/DCMI spec

#define GET_PWR_READING_MIN_RSPLEN	18	//Min Response length for Get Power Reading command
#define GET_PWR_READING_MAX_RSPLEN	18	//Max Response length for Get Power Reading command

//#define GET_PWR_LIMIT_MIN_RSPLEN	14	//Min Response length for Get Power Limit command
#define GET_PWR_LIMIT_MIN_RSPLEN	1	//Min Response length for Get Power Limit command
#define GET_PWR_LIMIT_MAX_RSPLEN	14	//Max Response length for Get Power Limit command

#define SET_PWR_LIMIT_MIN_RSPLEN	0	//Min Response length for Set Power Limit command
#define SET_PWR_LIMIT_MAX_RSPLEN	1	//Max Response length for Set Power Limit command

#define ACT_PWR_LIMIT_MIN_RSPLEN	1	//Min Response length for Activate Power Limit command
#define ACT_PWR_LIMIT_MAX_RSPLEN	1	//Max Response length for Activate Power Limit command

#include "DCMIClasses.hpp"


/**
 *
 */
class PwrMgmtTest: public DCMIClasses::DCMIBaseTest
{

	s_Opt_Platform_Attrib &s_optattrib;
	byte_t m_power;

#pragma pack(1)

/*
 * struct s_getPwrReading defines the Response data for GetPowerReading command
 * corresponding to table 6-8 in the DCMI 1.0 spec
 */
struct s_getPwrReading
{
//	byte_t  compCode;
//	UINT8	GrpExtID;
	UINT16	CurPwr;
	UINT16	MinPwr;
	UINT16	MaxPwr;
	UINT16	AvgPwr;
	UINT32	IPMITimeStamp;
	UINT32	StatTimePeriod;
	UINT16	PwrRdngState;
};

/*
 * struct s_getPwrLimit defines the Response data for GetPowerLimit command
 * corresponding to table-6-9 in the DCMI 1.0 spec
 * */
struct s_getPwrLimit
{
	byte_t  Active;
	UINT16  Rsvd1;
	UINT8   ExceptionAction;
	UINT16  PowerLimitReq;
	UINT32  CorrTime;
	UINT16  Rsvd2;
	UINT16  SampPeriod;
};

#pragma pack()

public:

   PwrMgmtTest (byte_t powerMgmt, s_Opt_Platform_Attrib &optattrib, TransportPlugin::TransportPlugIn &transport) :
	    DCMIBaseTest("DCMI Discovery for Power Management Controller Info", transport),
	    s_optattrib(optattrib), m_power(powerMgmt)
	{
//	   if (m_power == 1)
//	   {
		  ADD_TEST(PwrMgmtTest::GetPwrReadingTest, "DCMI Get Power Reading", TestUtil::TT_OPTIONAL);
		  ADD_TEST(PwrMgmtTest::GetPwrLimitTest,   "DCMI Get Power Limit",   TestUtil::TT_OPTIONAL);
//	   } else
//		   ADD_INFO(PwrMgmtTest::PwrMgmtInfo);
	}

   TestResult PwrMgmtInfo(std::ostream &stream)
   {
	   stream<<"Power Management is NOT Supported and it is Optional.";
	   return TestUtil::TR_SKIPPED;
   }

   TestResult GetPwrReadingTest(any_value &);
   TestResult GetPwrLimitTest(any_value &);

protected:

   s_getPwrReading getPowerReading();
   s_getPwrLimit getPowerLimit();
};

