/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*******************************************************************************
***		 File		:	DCMI_SEL_lib.h		  								  **
***																			  **
********************************************************************************/
//Define the Response length expected as in IPMI/DCMI spec

#define	GET_SEL_INFO_MIN_RSPLEN	14	//Min Response length for Get SEL Info command
#define	GET_SEL_INFO_MAX_RSPLEN	14	//Max Response length for Get SEL Info command

#define GET_SEL_ENTRY_MIN_RSPLEN 2	//Min Response length for Get SEL Entry command
#define GET_SEL_ENTRY_MAX_RSPLEN 18	//Max Response length for Get SEL Entry command

#define	RESERVE_SEL_MIN_RSPLEN	 2	//Min Response length for Reserve SEL command
#define	RESERVE_SEL_MAX_RSPLEN	 2	//Max Response length for Reserve SEL command

#define CLEAR_SEL_MIN_RSPLEN	 1	//Min Response length for Clear SEL command
#define CLEAR_SEL_MAX_RSPLEN	 1	//Max Response length for Clear SEL command



#include "DCMIClasses.hpp"

#pragma pack(1)
//The following structure defines the 16 bytes of SEL entry returned in
//Get Sel Entry command
struct s_sel_entry
{
   UINT16   iNextRecID;
   UINT16   iRecID;
   byte_t   bRecType;
   UINT32   iTimeStamp;
   UINT16   iGenID;
   byte_t   bEventMsgRev;
   byte_t   bSensorType;
   byte_t   bSensorNum;
   byte_t   bEventType;
   byte_t   bEventData1;
   byte_t   bEventData2;
   byte_t   bEventData3;
};

//s_getselentry_rsp defines the response data of Get SEL Entry command
struct s_getselentry_rsp
{
   int iCompCode;
   s_sel_entry  sSelRecord;
};

//defines the response data of Reserve SEL command response
struct s_reserve_sel_rsp
{
   byte_t   iCompCode;
   byte_t   iRidLsb;
   byte_t   iRidMsb;
};

#pragma pack()


using TestUtil::TestResult;

class SELTest: public DCMIClasses::DCMIBaseTest
{
	int m_SelEntries;

	struct record_id
	{
		byte_t rLsb;
		byte_t rMsb;

		record_id(byte_t l, byte_t m):
			rLsb(l), rMsb(m)
		{};
	};

	struct reserv_id
	{
		byte_t ridLsb;
		byte_t ridMsb;
	};

	reserv_id m_ResId;

	//timeouts in seconds
	int m_TimeOutSELReserve;
	int m_TimeOutSELGet;
	int m_TimeOutSELClear;

public:
   SELTest(TransportPlugin::TransportPlugIn &transport) :
      DCMIBaseTest("SEL Tests", transport)
   {
	   m_TimeOutSELReserve = 10;
	   m_TimeOutSELGet = 10;
	   m_TimeOutSELClear = 10;

	   ADD_TEST(SELTest::GetSelInfoTest, "Get SEL Info", TestUtil::TT_MANDATORY);
	   ADD_TEST(SELTest::ReserveSelTest, "Reserve SEL", TestUtil::TT_MANDATORY);
	   ADD_TEST_PARAM(SELTest::GetReservSelEntryTest, record_id(0,0), "Get SEL Entry, with Reservation ID", TestUtil::TT_OPTIONAL);

	   ADD_TEST_PARAM(SELTest::GetSelEntryTest, record_id(0,0), "Get First SEL Entry after Reservation", TestUtil::TT_OPTIONAL);
	   ADD_TEST_PARAM(SELTest::GetSelEntryTest, record_id(0xff,0xff), "Get Last SEL Entry and Verify", TestUtil::TT_OPTIONAL);

	   ADD_TEST(SELTest::ClearSelTest, "Clear SEL", TestUtil::TT_MANDATORY);
	   ADD_TEST(SELTest::VerifyClearSelTest, "Verify SEL Clear Action", TestUtil::TT_MANDATORY);
   }

   void doConfigure(DcmiConfiguration &option);

 /*tests ...*/
   TestResult GetSelInfoTest(any_value &);
   TestResult ReserveSelTest(any_value &);
   TestResult GetSelEntryTest(any_value &an_record);
   TestResult GetReservSelEntryTest(any_value &an_record);
   TestResult ClearSelTest(any_value &);
   TestResult VerifyClearSelTest(any_value &);

protected:

   /*utils ...*/

   int getSelInfo();
   reserv_id getReserveSel();
   s_sel_entry getSelEntry(reserv_id resid, record_id recid);
   void clearSel(reserv_id resid, byte_t clrAction, std::vector<byte_t> &rsp_dest);

};
