/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*******************************************************************************
***		 File		:	DCMI_SOL_lib.h    								      **
***																			  **
********************************************************************************/
#ifndef _DCMI_SOL_LIB_H
#define _DCMI_SOL_LIB_H


//Define the Response length expected as in IPMI/DCMI spec
#define GET_SOL_CONFIG_MIN_RSPLEN	0	//Min Response length for Get Serial/Modem Config Command
#define GET_SOL_CONFIG_MAX_RSPLEN	11	//Max Response length for Get Serial/Modem Config Command
						 //for OEM 255

#define SET_SOL_CONFIG_MIN_RSPLEN	0	//Min Response Length for Set Serial/Modem Config command
#define SET_SOL_CONFIG_MAX_RSPLEN	0	//Max Response Length for Set Serial/Modem Config command

#define GET_CHANNEL_AUTH_MIN_RSPLEN	 8	//Min Response length for Get Serial/Modem Config Command
#define GET_CHANNEL_AUTH_MAX_RSPLEN	 8	//Max Response length for Get Serial/Modem Config Command

#define GET_PAYLOAD_MIN_RSPLEN 3
#define GET_PAYLOAD_MAX_RSPLEN 3

#define GET_PAYLOAD_INF_MIN_RSPLEN 12
#define GET_PAYLOAD_INF_MAX_RSPLEN 12

#define GET_PAYLOAD_SUP_MIN_RSPLEN 8
#define GET_PAYLOAD_SUP_MAX_RSPLEN 8

#include "parameters.h"
#include "DCMI_VLAN_lib.h"

#define IPMI_1_5_AUTH_NONE     0x01
#define IPMI_1_5_AUTH_MD2      0x02
#define IPMI_1_5_AUTH_MD5      0x04
#define IPMI_1_5_AUTH_PASSWORD 0x10
#define IPMI_1_5_AUTH_OEM      0x20


class SOLTest: public LanBaseTest //public DCMIClasses::DCMIBaseTest
{
		//Define struct for capture of the original system Serial Over Lan parameters
	struct s_Sol_Orig_Params
	{
		int iSetInProg;
		int iSolEnable;
		int iSolAuth;
		int iCharAccum[2];
		int iSolRetry[2];
		int iSolNVBitRate;
		int iSolVolBitRate;
	};

	// Authentication capabilities response structure as IPMI Spec
	struct s_Channel_Auth_Cap
	{
		byte_t channel_number;

		byte_t v20_data_available : 1; /* IPMI v2.0 data is available */
		byte_t reserved1          : 1;
		byte_t v15_auth_types     : 6; /* IPMI v1.5 enabled auth types */

		byte_t reserved2          : 2;
		byte_t kg_status          : 1; /* two-key login status */
		byte_t per_message_auth   : 1; /* per-message authentication status */
		byte_t user_level_auth    : 1; /* user-level authentication status */

		byte_t non_null_usernames : 1; /* one or more non-null users exist */
		byte_t null_usernames     : 1; /* one or more null usernames non-null pwds */
		byte_t anon_login_enabled : 1; /* a null-named, null-pwd user exists */

		byte_t reserved3          : 6;
		byte_t ipmiv15_support    : 1; /* channel supports IPMI v1.5 connections */
		byte_t ipmiv20_support    : 1; /* channel supports IPMI v2.0 connections */

		byte_t oem_id[3];    /* IANA enterprise number for auth type */
		byte_t oem_aux_data; /* Additional OEM specific data for oem auths */
	};
	//__attribute__ ((packed));

	struct s_Payload_Activation
	{
		byte_t reserved 		: 4;
		byte_t num_instances	: 4;

		byte_t instances[2]		;
	};

	struct s_Payload_Info
	{
		byte_t sess_id[4];
		byte_t port_num;
		byte_t reserved[7];
	};


	struct s_Payload_Support
	{
		byte_t standard_payload[2];
		byte_t session_setup[2];
		byte_t oem_payload[2];
		byte_t reserved [2];
	};


public:
	SOLTest(byte_t channel, TransportPlugin::TransportPlugIn &transport) :
		LanBaseTest(channel, "DCMI SOL Tests", transport)
	{
		ADD_TEST(SOLTest::SolConfigTest, "Checking Serial Over Lan Configuration", TestUtil::TT_MANDATORY);
		ADD_TEST_PARAM(SOLTest::SolChannelAuthCapabilitiesTest, (PRIV_ADMIN), "Checking SOL Channel Auth. Capabilities", TestUtil::TT_MANDATORY);
		ADD_TEST_PARAM(SOLTest::SolPayloadActivationStatusTest, byte_t(0x01), "Checking SOL Payload Activation - Type SOL", TestUtil::TT_MANDATORY);
		ADD_TEST_PARAM(SOLTest::SolPayloadInstanceInfoTest,  (short)(0x101), "Checking SOL Payload Instance Info - Type SOL", TestUtil::TT_MANDATORY);
	//	ADD_TEST(SOLTest::SolPayloadSupportTest,  "Checking SOL Payload Support", TestUtil::TT_MANDATORY);
	}

	TestResult	SolConfigTest(any_value &);
	TestResult  SolChannelAuthCapabilitiesTest(any_value &);
	TestResult  SolPayloadActivationStatusTest(any_value &);
	TestResult  SolPayloadInstanceInfoTest(any_value &);
	TestResult  SolPayloadSupportTest(any_value &);

protected:

	void getSolConfig(int iSolParam, int iSetSelect, int iBlockSelect, std::vector<byte_t> &resp_data);

	struct s_Payload_Activation getSolPayloadActivationStatus(byte_t num);

	struct s_Channel_Auth_Cap getSolChannelAuthCapabilities(PrivelegeLevel level);

	struct s_Sol_Orig_Params getSolOrigValues(int iSetSelect, int iBlockSelect);

	struct s_Payload_Info getSolPayloadInstanceInfo(byte_t typ_num, byte_t inst_num);

	struct s_Payload_Support getSolPayloadSupport(byte_t chann_num);

};

#endif //_DCMI_SOL_LIB_H
