/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*******************************************************************************
***		 File		:	DCMI_Sensor_lib.h									  **
***																			  **
********************************************************************************/
//Define the Response length expected as in IPMI/DCMI spec

#define DCMI_SENSOR_INFO_MIN_RSPLEN	 3		//Minimum expected response length for Get DCMI Sensor Info Command
#define DCMI_SENSOR_INFO_MAX_RSPLEN	 (8*2) + DCMI_SENSOR_INFO_MIN_RSPLEN		//Maximum expected response length for Get DCMI Sensor Info Command

#define GET_SENSOR_RDNG_MIN_RSPLEN	2	//Min response length for Get Sensor Reading command
#define GET_SENSOR_RDNG_MAX_RSPLEN	7	//Max response length for Get Sensor Reading command


class SensorTest: public SDRBase
{
	struct record_id
	{
		byte_t rLsb;
		byte_t rMsb;

		record_id(byte_t l, byte_t m):
			rLsb(l), rMsb(m)
		{};
	};

	struct reserv_id
	{
		byte_t ridLsb;
		byte_t ridMsb;
	};

	//Define the struct for Sensor Discovery
	struct s_sensor
	{
	    byte_t sensorType;
	    byte_t entityId;
	    byte_t entityInstance;
	    byte_t entityStart;
	};

public:
	SensorTest(TransportPlugin::TransportPlugIn &transport) :
		SDRBase("DCMI Sensor Tests", transport)
	{
		ADD_TEST_PARAM(SensorTest::EntitySensorsTest, byte_t(0x40), "Sensors Entity: Inlet     (0x40)", TestUtil::TT_MANDATORY );
		ADD_TEST_PARAM(SensorTest::EntitySensorsTest, byte_t(0x41), "Sensors Entity: CPU       (0x41)", TestUtil::TT_MANDATORY );
		ADD_TEST_PARAM(SensorTest::EntitySensorsTest, byte_t(0x42), "Sensors Entity: Baseboard (0x42)", TestUtil::TT_MANDATORY );
	}

	TestResult EntitySensorsTest(any_value &param)
	{
		/*
		 *  Sensor Reading has to be compared to be within limits only if UUT is in DC ON state
		 *  Check for Chassis support in DCMI Capability command, and issue Chassis Status
		 */
		byte_t entityid = param.value<byte_t>();

		if (isRemoteConnection())
		{ //Remote connection check if UUT is in DC ON state
			//Check current state of system
			if (((getChassisStatus() & 0x01) == 0))
			{
				LOG_WARN<<"\n\tSystem is in DC OFF state"
						<<"\n\tEntity Sensor ID " <<std::hex<<std::showbase<<int(entityid)
						<<" and associated Sensors, SDRs cannot be tested!!";

				return TestUtil::TR_NOT_SUPPORTED;
			}
		}

		LOG_DEBUG<<"\n\tSystem is in DC ON state, verifying sensor entity...";
		verifySensor(entityid, 0x01);

		return TestUtil::TR_SUCCESS;
	};

protected:

	void sensorDiscovery(s_sensor discCmdData, byte_t &instanceRet, std::vector<record_id> &rec_ids);

	void verifySensor(byte_t entity_id, byte_t entity_nr);
	int getSensorReading(int sensorNum);
	void sensorThresholdTest(int sensorNum);

	TestResult SingleSensorTest(any_value &an_sensor);
};
