/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2011 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*
 * DCMI_Thermal_lib.h
 *
 *  Created on: 10-03-2011
 */

#ifndef DCMI_THERMAL_LIB_H_
#define DCMI_THERMAL_LIB_H_

#include "DCMIClasses.hpp"

//Define the Response length expected as in IPMI/DCMI spec

#define DCMI_THERMAL_INFO_MIN_RSPLEN	5		//Minimum expected response length for Get DCMI THERMAL Info Command
#define DCMI_THERMAL_INFO_MAX_RSPLEN	256		//Maximum expected response length for Get DCMI THERMAL Info Command

#define GET_THERMAL_RDNG_MIN_RSPLEN		3	//Min response length for Get THERMAL Reading command
#define GET_THERMAL_RDNG_MAX_RSPLEN		8*2 + GET_THERMAL_RDNG_MIN_RSPLEN 	//Max response length for Get THERMAL Reading command

#define GET_THERMAL_MIN_RSPLEN		5	//Min response length for Get THERMAL Limit command
#define GET_THERMAL_MAX_RSPLEN		256	//Max response length for Get THERMAL Limit command

#define SET_THERMAL_MIN_RSPLEN	1	//Min response length for Set THERMAL Limit command
#define SET_THERMAL_MAX_RSPLEN  256	//Max response length for Set THERMAL Limit command


using TestUtil::TestResult;

class ThermalTest: public DCMIClasses::DCMIBaseTest
{
	static const byte_t temp_limit = 30;
	static const short exception_time = 660;

public:
	ThermalTest(TransportPlugin::TransportPlugIn &transport) :
		DCMIBaseTest("Thermal Management Tests", transport)
	{
		ADD_TEST_PARAM(ThermalTest::ThemperatureReadingTest, byte_t(0x40), "Temperature reading for sensor: Inlet     (0x40)", TestUtil::TT_MANDATORY );
		ADD_TEST_PARAM(ThermalTest::ThemperatureReadingTest, byte_t(0x41), "Temperature reading for sensor: CPU       (0x41)", TestUtil::TT_MANDATORY );
		ADD_TEST_PARAM(ThermalTest::ThemperatureReadingTest, byte_t(0x42), "Temperature reading for sensor: Baseboard (0x42)", TestUtil::TT_MANDATORY );
//		ADD_TEST_PARAM(ThermalTest::GetThermalLimitTest,	 byte_t(0x40), "Get Thermal limit for sensor: Inlet  (0x40)", TestUtil::TT_OPTIONAL);
//		ADD_TEST_PARAM(ThermalTest::SetThermalLimitTest,	 byte_t(0x40), "Set Thermal limit for sensor: Inlet  (0x40)", TestUtil::TT_MANDATORY );
	};

	TestResult ThemperatureReadingTest(any_value &param)
	{
		themperatureReading(param.value<byte_t>(),0x01);
		return TestUtil::TR_SUCCESS;
	};

	TestResult GetThermalLimitTest(any_value &param)
	{
		getThermalLimit(param.value<byte_t>(),0x01);
		return TestUtil::TR_SUCCESS;
	};

	TestResult SetThermalLimitTest(any_value &param)
	{
		setSessionPrivilegeLevel(PRIV_OPERATOR); //minimum privilege
		setThermalLimit(param.value<byte_t>(),0x01);
		return TestUtil::TR_SUCCESS;
	};


protected:

	struct temperature_data
	{
		byte_t value :7;
		byte_t sign :1;
		byte_t idx;
	} ;

	struct exception_action
	{
		byte_t reserved :5; //4:0
		byte_t log :1; //5
		byte_t poweroff :1; //6
		byte_t persist :1; //7
	} ;

	struct thermal_info
	{
		exception_action action;
		byte_t limit;
		short exTime;
	};

	void themperatureReading(byte_t entity_id, byte_t entity_nr);
	thermal_info getThermalLimit(byte_t entity_id, byte_t entity_nr);
	void setThermalLimit(byte_t entity_id, byte_t entity_nr);

};

#endif /* DCMI_THERMAL_LIB_H_ */
