/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*******************************************************************************
***		 File		:	DCMI_VLAN_lib.h  									  **
***																			  **
********************************************************************************/

#ifndef _DCMI_VLAN_LIB_H
#define _DCMI_VLAN_LIB_H

//Define the Set/Get LAN Configuration Parameters
#define MAC_ADDR_PARAM			5	//MAC Address Parameter index of SET/GET LAN Config Param cmd
#define ARP_CTRL_PARAM			10  //BMC-generated ARP control Param
#define IP_SRC_PARAM			4	//IP source
#define VLANID_PARAM			20	//VLANID Parameter index of SET/GET LAN Config Param cmd
#define VLAN_PRIORITY_PARAM		21	//VLAN Priority Parameter index of SET/GET LAN Config Param cmd
#define VLAN_RMCPP_ENTRY_SUPP	22	//VLAN RMCPP+ Messaging Cipher Suite Entry Support Min
#define VLAN_RMCPP_ENTIRES		23	//VLAN RMCPP+ Messaging Cipher Suite Entry Support ID
#define VLAN_RMCPP_PRIV_LEVEL	24	//VLAN VLAN RMCPP+ Messaging Cipher Suite Entry Support Level

#define VLAN_TAG_PARAM			25 	//VLAN Tag Parameter index of SET/GET LAN Config Param cmd

#define NUM_DESTINATION         17 	//Num of Destn Parameter index of SET/GET LAN Config Param cmd
#define VLAN_TEST_ID            0x413 //sample VLAN ID used in the Test for validating VLAN support


//Define the Response length expected as in IPMI/DCMI spec

#define GET_LAN_CONFIG_MAX_RSPLEN	18	//Max Response length for Get LAN Config Command
#define GET_LAN_CONFIG_MIN_RSPLEN	2	//Min Response length for Get LAN Config Command


#define GET_SESSION_INFO_MIN_RSPLEN 7
#define GET_SESSION_INFO_MAX_RSPLEN 20

#define GET_LAN_CHANNEL_MAX_RSPLEN	20	//Max Response length for Get LAN Config Command
#define GET_LAN_CHANNEL_MIN_RSPLEN	0	//Min Response length for Get LAN Config Command

#define SET_LAN_CONFIG_MIN_RSPLEN	0	//Min Response Length for Set LAN Config command
#define SET_LAN_CONFIG_MAX_RSPLEN	0	//Max Response Length for Set LAN Config command

#define MCID_MAX_LEN      64       //Maximum length of characters in MC ID String is 64

#define GET_MCID_MAX_BYTES_TO_READ 16  //Get MC ID String command allows reading
                              // 16 bytes at a time
#define GET_MCID_MIN_RSPLEN   02    //Minimum expected response length for Get MC ID String Command
#define GET_MCID_MAX_RSPLEN   18    //Maximum expected response length for Get mc id Command

#define GET_LAN_CHANNEL_INFO_MAX_RSPLEN	 9	//Max Response length for Get LAN channel info Command
#define GET_LAN_CHANNEL_INFO_MIN_RSPLEN	 9	//Min Response length for Get LAN channel info Command

#define GET_USER_NAME_MIN_RSPLEN	 0	//Max Response length for Get user name Command
#define GET_USER_NAME_MAX_RSPLEN	 16	//Min Response length for Get user name Command

#define GET_USER_ACCESS_MIN_RSPLEN	 4	//Max Response length for Get user access Command
#define GET_USER_ACCESS_MAX_RSPLEN	 4	//Min Response length for Get user access Command

#define GET_USER_P_ACCESS_MIN_RSPLEN	 4	//Max Response length for Get user access Command
#define GET_USER_P_ACCESS_MAX_RSPLEN	 4	//Min Response length for Get user access Command

#include "DCMIClasses.hpp"

/**
 *
 */
class LanBaseTest: public DCMIClasses::DCMIBaseTest
{

protected:

   int m_channelNum;

	LanBaseTest(byte_t channel, std::string desc, TransportPlugin::TransportPlugIn &transport) :
		DCMIBaseTest(desc, transport)
	{
		/* The channel number will remain constant throughout */
		m_channelNum = channel;
	}

	void getLANConfig (byte_t param, std::vector<byte_t> &resp_data);

	int getChannelNum() { return m_channelNum; };
};

/****************************/

class VLanTest: public LanBaseTest
{

public:

	VLanTest(byte_t channel, TransportPlugin::TransportPlugIn &transport) :
		LanBaseTest(channel, "Verify support for the LAN Configuration Commands", transport)
	{
		//1. VLANID - parameter 20
		ADD_TEST(VLanTest::VLANIDTest, "VLAN Support Test", TestUtil::TT_MANDATORY);
		//2. VLAN Priority - parameter 21
		ADD_TEST(VLanTest::VLANPriorityTest, "VLAN Priority Test", TestUtil::TT_MANDATORY);
		//3. RMCPP+ Messaging Cipher Suite Entry Support - parameter 22
		ADD_TEST(VLanTest::RMCPPSupportTest, "VLAN RMCPP+ Entry Support", TestUtil::TT_MANDATORY);
		//4. VLAN RMCPP+ Messaging Cipher Suite Entries - parameter 23
		ADD_TEST(VLanTest::RMCPPEntiresTest, "VLAN RMCPP+ Entries ", TestUtil::TT_MANDATORY);
		//5. VLAN RMCPP+ Messaging Cipher Suite Privilege Level- parameter 24
		ADD_TEST(VLanTest::RMCPPPrivLevelTest, "VLAN RMCPP+ Privilege level", TestUtil::TT_MANDATORY);
	}

	//void doConfigure(DcmiConfiguration &option) ;

	TestResult VLANIDTest(any_value &);
	TestResult VLANPriorityTest (any_value &);

	TestResult VLANTagTest (any_value &);

	TestResult RMCPPSupportTest(any_value &);
	TestResult RMCPPEntiresTest(any_value &);
	TestResult RMCPPPrivLevelTest(any_value &);

};

/****************************/

class LanTest: public LanBaseTest
{
	std::string m_MCIDString;


	/*
	 * The GET USER ACCESS response based on table 22-32 of the IPMI v2.0 spec
	 */
	struct s_User_Access
	{
		byte_t reserved1    : 2;
		byte_t max_user_ids : 6;

		byte_t reserved2          : 2;
		byte_t enabled_user_count : 6;

		byte_t reserved3        : 2;
		byte_t fixed_name_count : 6;

		byte_t reserved4               : 1;
		byte_t no_callin_access        : 1;
		byte_t link_auth_access        : 1;
		byte_t ipmi_messaging_access   : 1;
		byte_t channel_privilege_limit : 4;
	};

	struct s_Standard_Payload {
		byte_t st_payloads : 7;
		byte_t reserved    : 1;
	};

	struct s_User_Payload_Access
	{
		struct s_Standard_Payload st_payload1;
		struct s_Standard_Payload st_payload2;

		byte_t oem_payload1;
		byte_t oem_payload2;
	};

public:

	LanTest(byte_t channel, TransportPlugin::TransportPlugIn &transport) :
		LanBaseTest(channel, "LAN Configuration Check Tests", transport)
	{
		 //1. VLANID - parameter 20
		ADD_TEST(LanTest::GratArpTest, "Lan Configuration Gratutious Arp Check", TestUtil::TT_MANDATORY);
		//2. VLAN Priority - parameter 21
		ADD_TEST(LanTest::TestArpCtrl, "Lan Configuration Arp Control Check", TestUtil::TT_MANDATORY);
		//3. RMCPP+ Messaging Cipher Suite Entry Support - parameter 22
		ADD_TEST(LanTest::IPsrcTest, "Lan Configuration IP Source Check", TestUtil::TT_MANDATORY);
		//4. VLAN RMCPP+ Messaging Cipher Suite Entries - parameter 23
		ADD_TEST(LanTest::AccessModeCheckTest, "Lan Configuration Access Mode Check", TestUtil::TT_MANDATORY);
		ADD_TEST(LanTest::UserAccessTest, "User Access Check", TestUtil::TT_MANDATORY);
		ADD_TEST(LanTest::UserPayloadAccessTest, "User Payload Access Check", TestUtil::TT_MANDATORY);
		//5. VLAN RMCPP+ Messaging Cipher Suite Privilege Level- parameter 24
		ADD_TEST(LanTest::MultiSessionTest, "Multi Session Test", TestUtil::TT_MANDATORY);
		ADD_TEST(LanTest::IssueGetMcidStringTest, "Get MC ID String Test", TestUtil::TT_MANDATORY);
		//	ADD_TEST(LanTest::McidNameTest, "DHCP Host String Discovery Test", TestUtil::TT_MANDATORY);
	}

//	void doConfigure(DcmiConfiguration &option) ;

	TestResult MultiSessionTest (any_value &);
	TestResult MACaddrTest(any_value &);
	TestResult GratArpTest(any_value &);
	TestResult TestArpCtrl(any_value &);
	TestResult IPsrcTest(any_value &);
	TestResult AccessModeCheckTest(any_value &);
	TestResult IssueGetMcidStringTest(any_value &);
	TestResult McidNameTest(any_value &);
	TestResult UserAccessTest(any_value &);
	TestResult UserPayloadAccessTest(any_value &);
protected:

	void getSessionInfo(std::vector<byte_t> &resp_data );
	void getChannelAccess(std::vector<byte_t> &resp_data);
	void getMcidString(byte_t Offset, byte_t NumBytes, std::vector<byte_t> &resp_data);

	void getChannelInfo(std::vector<byte_t> &resp_data);
	std::string getUserName(byte_t user_id);
	struct s_User_Access getUserAccess(byte_t user_id);
	struct s_User_Payload_Access getUserPayloadAccess(byte_t user_id);
};


#endif //_DCMI_VLAN_LIB_H
