/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2011 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*
 * DCMI_Wdt_lib.cpp
 *
 *  Created on: 02-05-2011
 *
 */

#ifndef DCMI_WDT_LIB_H_
#define DCMI_WDT_LIB_H_

#include "DCMIClasses.hpp"

#define GET_WDT_MIN_RSPLEN		8	//Min response length for Get WDT command
#define GET_WDT_MAX_RSPLEN		9	//Max response length for Get WDT command

/**
 *  Watchdog Timer test class
 */
class WatchdogTimerTest: public DCMIClasses::DCMIBaseTest
{

public:

	WatchdogTimerTest(TransportPlugin::TransportPlugIn &transport) :
			DCMIBaseTest("Watchdog Timer Tests", transport)
	{
		if (isRemoteConnection()) //optional test in OOB skipped
			return ;

   //     ADD_TEST(WatchdogTimerTest::ResetWatchdogTest, "Reset Watchdog Timer", TestUtil::TT_MANDATORY);
		ADD_TEST(WatchdogTimerTest::GetWatchdogTest,   "Get Watchdog Timer",   TestUtil::TT_MANDATORY);
//		ADD_TEST_PARAM(WatchdogTimerTest::SetThermalLimitTest,	 byte_t(0x40), "Set Thermal limit for sensor: Inlet  (0x40)", TestUtil::TT_MANDATORY );
	};

	TestResult ResetWatchdogTest( any_value &)
	{
		LOG_SUMMARY<<"\nReset Watchdog Timer command";
		setSessionPrivilegeLevel(PRIV_OPERATOR);
		resetWatchdogTimer();
		return TestUtil::TR_SUCCESS;
	}

	TestResult GetWatchdogTest( any_value &)
	{
		LOG_SUMMARY<<"\nGet Watchdog Timer command";
		getWatchdogTimer();
		return TestUtil::TR_SUCCESS;
	}

protected:

	struct timer_use
	{
		byte_t donot_log: 1;
		byte_t donot_stop: 1;
		byte_t reserv: 3;
		byte_t use: 3;
	} ;

	struct  timer_action
	{
		byte_t reserv_7: 1;
		byte_t pre_timeout: 3;
		byte_t reserv_3: 1;
		byte_t action: 3;
	} ;

	struct timer_expiration
	{
		byte_t reserv_7_6: 2;
		byte_t oem:1;
		byte_t sms_os:1;
		byte_t os_load:1;
		byte_t bios_post :1;
		byte_t bios_frb2:1;
		byte_t reserv_0: 1;
	} ;

	struct wdt_timer_main
	{
		struct timer_use use;
		struct timer_action action;
		byte_t pre_timout;
		struct timer_expiration expiration;
		byte_t init_countdown_lsb;
		byte_t init_countdown_msb;
	} ;

	struct wdt_timer : public wdt_timer_main
	{
		byte_t present_countdown_lsb;
		byte_t present_countdown_msb;
	} ;

	void resetWatchdogTimer() ;
	wdt_timer getWatchdogTimer();

};

#endif /* DCMI_WDT_LIB_H_ */
