/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2011 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intel�s	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*
 * any_value.hpp
 *
 *  Created on: 10-04-2011
 */

#ifndef ANY_VALUE_HPP_
#define ANY_VALUE_HPP_

#include <stdexcept>
#include <memory>
#include <typeinfo> 


namespace CommonClasses
{

class any_value
{
	class storage
	{
	public:
		virtual const void *value() = 0;
		virtual const std::type_info & type() = 0;
		virtual storage *clone() = 0;
	};

   template <typename T>
   class storage_impl : public storage
	{
	public:
		T m_value;

		storage_impl(const T & rhs) :
			m_value(rhs)
		{}

		const std::type_info & type()
		{
			return typeid(m_value);
		}

		const void *value()
		{
			return &m_value;
		}

		storage *clone()
		{
			return new storage_impl<T>(m_value);
		}
	};

   //here must be a real class can not be a template
   //so need to make fake abstract class and then do an template
	std::auto_ptr<storage> m_value;

public:
	// construct/copy/destruct
	any_value() {};

	~any_value()
	{
		m_value.release();
	}

	any_value(const any_value &rhs)
	{
		operator=(rhs);
	}

	any_value &operator=(const any_value &rhs)
	{
		m_value.reset(rhs.empty()?NULL:rhs.m_value->clone());
		return *this;
	}

	template<typename T>
	any_value(T rhs)
	{
		operator=(rhs);
	}

	template<typename T>
	any_value &operator=(const T &rhs)
	{
        m_value.reset(new storage_impl<T>(rhs));
		return *this;
	}

	bool empty() const
	{
		return (m_value.get()==NULL);
	}

	template <typename T>
	const T &value() /*throw(...)*/
	{
		if (empty())
			std::runtime_error("Any value is empty");
		if (typeid(T) != m_value->type())
			std::runtime_error(std::string("Different type of any value : ").append(m_value->type().name()));

		return *static_cast<const T *>(m_value->value());
	}

	const std::type_info & type() const
	{
		return (m_value.get()!= NULL ? m_value->type() : typeid(void));
	}

};


}//CommonClasses

#endif /* ANY_VALUE_HPP_ */
