/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*******************************************************************************
***		 File		:	Parameters.h    									  **
***																			  **
********************************************************************************/

#ifndef _PARAMETERS_H
#define _PARAMETERS_H

#ifdef WIN32

#else
    #include <curses.h>
    #include <ncurses.h>
#endif

#include "transport.h"
#include "Logger.h"


//Following definitions are for return status from IPMI commands / functions
//Transport Error caused by network failure or Packet loss

//APP_ERROR: This enum defines the return values for run()
enum APP_ERROR
{
   SUCCESS = 0,
   E_TRANSPORT	    =	-1,
   E_SAT_PWR_CTRLR	=	-2,
   E_CONFIG_LOAD	=	-3,
   E_INIT_LOGGER	=	-4,
   E_INVALID_OPTION	=	-5,
   E_FAILURE = -6
};

//The User Options are as follows
enum UserOption
{
   U_INVALID = -1,
   U_EXIT = 0,
   OOB_RMCPP = 1,
   IB_KCS_HECI,
   CIPHER_CHECK
};

enum transport_plugin_id
{
	TP_RMCPP,
	TP_INB
};

//Privelege Level of Commands
enum PrivelegeLevel
{
	PRIV_UNSPECIFIED,
	PRIV_CALLBACK	= 0x01,
	PRIV_USER		= 0x02,
	PRIV_OPERATOR	= 0x03,
	PRIV_ADMIN		= 0x04,
	PRIV_OEM 		= 0x05
};

//Define the DCMI Extension ID
//#define DCMI_EXT_ID  0xDC

//Defining the netfunctions to be used in the DCMI Conformance Test Suite
enum DCMI_NET_FN
{

    CHASSIS = 0x00,
    APPLICATION = 0x06,
    SENSOR = 0x04,
    STORAGE = 0x0A,
    TRANSPORT = 0x0C,
    DCMI = 0x2C
};


typedef unsigned char  UINT8;
typedef unsigned short UINT16;
typedef unsigned int /*unsigned long*/  UINT32;

#pragma pack(1)

//Define the struct for return data from issuing the DCMI Discovery Command
struct s_sensor_discovery
{
  //  byte_t identificationId;
    byte_t sensorType;
    byte_t entityId;
    byte_t entityInstance;
    byte_t entityStart;
};



//From the record ID pulled from the DCMI discovery command will need
//to load the following parameters for checking of sensor readings and
//comparing to the upper and lower sensor values.
struct s_sdr_data
{

	byte_t sensorType;
	byte_t senseNumber;
    byte_t senseUnits;
    byte_t unrValue;
    byte_t ucValue;
    byte_t uncValue;
    byte_t lnrValue;
    byte_t lcValue;
    byte_t lncValue;
    byte_t readMaskValue;

};

//Define the Mandatory Platform Capabilities returned in
//"Get DCMI Capability Info" command for Parameter 1
//corresponding to Table 6-3 of DCMI Spec ver 1.0
//spec 1.1/1.5 makes this byte reserved, finally here we have different purpose for this byte
struct s_min_platform_capab
{
	UINT8 bIDSupport:1 ;
	UINT8 bSELLog:1 ;
	UINT8 bChassisPower:1 ;
	UINT8 bTempMonitor:1 ;
	UINT8 Reserved:4 ;
};

//Define the Optional Platform Capabilities returned in
//"Get DCMI Capability Info" command for Parameter 1
// corresponding to Table 6-3 of DCMI Spec ver 1.0

struct s_opt_platform_capab
{
	UINT8 bPowerMgmt:1;
	UINT8 Reserved:7;
};

//Define the Manageability access Capabilities returned in
//"Get DCMI Capability Info" command for Parameter 1
// corresponding to Table 6-3 of DCMI Spec ver 1.1
struct s_mgmt_access_capab
{
	UINT8 bKCSpresent:1;
	UINT8 bOOBTmodePresent:1;
	UINT8 bOOBSecLanPresent:1;
//	UINT8 bOOBPrimLanPresent:1;
//	UINT8 bSOLpresent:1;
//	UINT8 bVLANpresent:1;
	UINT8 Reserved:5;
};

//Define the Identification Attributes returned in
//"Get DCMI Capability Info" command for Parameter 2
// corresponding to Table 6-3 of DCMI Spec ver 1.1
/*
struct s_ID_Attrib
{
	UINT8 bGUIDSupport:1 ;
	UINT8 bDHCPHostName:1 ;
	UINT8 bAssetTagSupport:1;
	UINT8 Reserved:5;
};
*/
struct s_ID_Attrib
{
	UINT8 Reserved;
};
//Define the Temperature monitoring returned in
//"Get DCMI Capability Info" command for Parameter 2
// corresponding to Table 6-3 of DCMI Spec ver 1.0
struct s_TempMonitor
{
	UINT8 bInletTemp:1 ;
	UINT8 bProcessorTemp:1;
	UINT8 bBaseboardTemp:1;
	UINT8 Reserved:5;
};

//Defining the return data for byte 6:8
//for GetDCMICapabilitiesInfo command - parameter Selector 1
//corresponds to Table6-3 of DCMI Spec ver 1.0
struct s_supported_DCMI_capab
{
	struct s_min_platform_capab smin_capab;
    struct s_opt_platform_capab sopt_capab;
    struct s_mgmt_access_capab  smgmt_capab;
};

//Defining the return data for byte 6:9
//for GetDCMICapabilitiesInfo command - parameter Selector 2
//corresponds to Table6-3 of DCMI Spec ver 1.0
struct s_Min_Platform_Attrib
{
    UINT8 uSELAttrib[2];
	struct s_ID_Attrib sID;
	struct s_TempMonitor sTempSensor;
	UINT8 uSamplingFreq;
};

//Defining the return data for byte 6:7
//for GetDCMICapabilitiesInfo command - parameter Selector 3
//corresponds to Table6-3 of DCMI Spec ver 1.0
struct s_Opt_Platform_Attrib
{
    UINT8 uPMDevSlaveAddr;
	UINT8 uPMChannelNum;
};

//Defining the return data for byte 6:8
//for GetDCMICapabilitiesInfo command - parameter Selector 4
//corresponds to Table6-3 of DCMI Spec ver 1.1
struct s_Mgmt_Access_Attrib
{
    UINT8 PrimChannelNum;
	UINT8 SecChannelNum;
	UINT8 TMODEChannelNum;
};

//Defining the return data for
//for GetDCMICapabilitiesInfo command - parameter Selector 5
//corresponds to Table6-3 of DCMI Spec ver 1.1
struct s_System_Power_Attrib
{
    UINT8 NumOfRollingPeriod;
	UINT8 RollingPeriods[128];
};
#pragma pack()


//Structure for SEL Capabilities
struct SEL_Cape
{
	int SEL_Auto_Roll_Enabled;		// SEL automatic rollover enabled
    int SEL_Entry;					// number of SEL entries
};


#ifndef WIN32
#define strcat_s(dest, destSize, src)\
    (strlen(dest) + strlen(src) + 1 > destSize) ? \
        (strlen(dest) + 1 < destSize) ? \
        strncat(dest, src, destSize - strlen(dest) - 1) : dest : strcat(dest, src)
#define	scanf_s scanf
#endif

#endif

