/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2011 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*
 * CommonClasses.cpp
 *
 *  Created on: 28-02-2011
 */
#include <string>
#include "parameters.h"
#include "CommonClasses.hpp"


using std::string;


namespace CommonClasses
{

void Response::print_header(std::ostream &stream)
{
	stream << "\n\tCommand Response: " << std::hex << std::showbase
			<< static_cast<int> (getCompCode()) ;
}

void Response::print(std::ostream &stream)
{
	print_header(stream);
	if (getLength())
	{
		stream << " ";
		std::copy(&at(0), &at(getLength()), std::ostream_iterator<int>((stream<<std::hex<<std::showbase), " "));
	}
}

void Request::print (std::ostream &stream)
{
	stream << "\n\tCommand Request: " <<std::hex<<std::showbase
//			<< static_cast<int> (rsAddr) << " "
			<< static_cast<int> (netFun) << " "
			<< static_cast<int> (rsLun)  << " "
			<< static_cast<int> (cmd);
	if (len)
	{
		stream << " ";
		std::copy(&data[0], &data[len], std::ostream_iterator<int>((stream<<std::hex<<std::showbase), " "));
	}
}

/*********************************************************************************
	Function Declaration:
	string getCompletionCodeDescryption(byte_t )
	Function Name:		getCompletionCodeDescryption
	Purpose:			Gives the Completion Code Descryption for the ipmi response code.
						The command name is passed as string to be printed.
	Parameters:		[IN] compCode - The completion code from IPMI response
	Returns:		descryption string
	********************************************************************************/
std::string Response::getCompletionCodeDescryption(byte_t compCode)
{
   std::string buff;
	switch(compCode)
	{
	case 0x00:
		buff ="Completed Normally";
		break;
	case 0x80:
		buff ="Parameter not supported";
		break;
	case 0x81:
		buff ="Something else has already claimed these parameters.";
		break;
	case 0x82:
		buff ="Not supported or failed to write a read-only parameter.";
		break;
	case 0x83:
		buff ="Access mode is not supported.";
		break;
	case 0x84:
		buff ="Power/Thermal limit out of range.";
		break;
	case 0x85:
		buff ="Correction/Exception time out of range.";
		break;
	case 0x89:
		buff ="Sample/Statistics Reporting period out of range.";
		break;
	case 0x8A:
		buff ="Power limit already active.";
		break;
	case 0xC0:
		buff ="Node Busy";
		break;
	case 0xC1:
		buff ="Invalid Command";
		break;
	case 0xC2:
		buff ="Command invalid for given LUN";
		break;
	case 0xC3:
		buff ="Timeout while processing command";
		break;
	case 0xC4:
		buff ="Out of space";
		break;
	case 0xC5:
		buff ="Invalid Reservation ID";
		break;
	case 0xC6:
		buff ="Request data truncated";
		break;
	case 0xC7:
		buff ="Request data length invalid";
		break;
	case 0xC8:
		buff ="Request data field length limit exceeded";
		break;
	case 0xC9:
		buff ="Parameter out of range";
		break;
	case 0xCA:
		buff ="Cannot return number of requested data bytes";
		break;
	case 0xCB:
		buff ="Requested Sensor, data, or record not present";
		break;
	case 0xCC:
		buff ="Invalid data field in Request";
		break;
	case 0xCD:
		buff ="Command illegal for specified sensor or record type";
		break;
	case 0xCE:
		buff ="Command response could not be provided";
		break;
	case 0xCF:
		buff ="Cannot execute duplicated request";
		break;
	case 0xD0:
		buff ="Command response could not be provided. SDR Repository in update mode";
		break;
	case 0xD1:
		buff ="Command response could not be provided.  Device in FW update mode";
		break;
	case 0xD2:
		buff ="Command response could not be provided. BMC initialization in progress";
		break;
	case 0xD3:
		buff ="Destination unavailable";
		break;
	case 0xD4:
		buff ="Cannot execute command due to insufficient privilege level";
		break;
	case 0xD5:
		buff ="Command not supported in present state.";
		break;
	case 0xD6:
		buff ="Cannot execute command. Parameter is illegal.";
		break;
	case 0xFF:
	default:
		buff ="Unspecified error.";
	};

	return buff;
}


void IpmiResponse::copy_to(size_t _offset, byte_t *_dest, size_t _size)
{
	if ((_offset) > getLength())
		EX_OutOfRange();

#ifdef _MSC_VER
	std::copy(&at(_offset), &at(std::min((_size + _offset), getLength())),
			stdext::checked_array_iterator<byte_t*>(_dest, _size));
#else
	std::copy(&at(_offset), &at(std::min((_size + _offset), getLength())), _dest);
#endif
}

void IpmiResponse::copy_to(size_t _offset, std::vector<byte_t> &_dest)
{
	if ((_offset) > getLength())
		EX_OutOfRange();
	_dest.resize((getLength()-_offset));

	std::copy(&at(_offset), &at(getLength()), _dest.begin());
}

}//namespace
