/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2011 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*
 * DCMI1_1ConformanceSuit.cpp
 *
 *  Created on: 04-04-2011
 */

#include "parameters.h"
#include "DCMI_Configuration.h"

#include "DCMI_Misc_lib.h"
#include "DCMI_SEL_lib.h"
#include "DCMI_SOL_lib.h"
#include "DCMI_Tmode_lib.h"
#include "DCMI_SDR_lib.h"
#include "DCMI_Chassis_lib.h"
#include "DCMI_VLAN_lib.h"
#include "DCMI_PwrMgmt_lib.h"
#include "DCMI_Sensor_lib.h"

#include "TransportManager.h"

#include "DCMI1_1ConformanceSuit.h"

using TransportPlugin::TransportManager;

namespace DCMIConformance
{


DCMI1_1ConformanceSuite::DCMI1_1ConformanceSuite(int ver):
		DCMIDiscovery(ver, TransportManager::getCurrentTransport())
{
	s_supported_DCMI_capab& s_DCMICapab = getCapbilities();
	s_Mgmt_Access_Attrib &s_MgmtAttrib  = getMgmtAccessAttrib();

	/**
	 * Discovery phase finished ok so ... add rest test cases..
	 *
	 */
	ADD_TEST(DCMIDiscovery::PlatformSuppCapabTest, "Supported DCMI Platform Capabilities", TestUtil::TT_MANDATORY);
	ADD_TEST(DCMIDiscovery::MgmtAccessAttribTest,  "Manageability Access Attributes", TestUtil::TT_MANDATORY);

	ADD_TEST(DCMIDiscovery::PlatformSuppCapabSessionLessTest, "Session Less Capabilities", TestUtil::TT_MANDATORY);
	ADD_TEST(DCMIDiscovery::MinPlatformAttribTest, "Minimum Platform Attributes", TestUtil::TT_MANDATORY);
	ADD_TEST(DCMIDiscovery::OptPlatformAttribTest, "Optional Platform Attributes", TestUtil::TT_OPTIONAL);
	ADD_TEST(DCMIDiscovery::SystemPowerAttribTest, "Enhanced System Power Statistics Attributes", TestUtil::TT_OPTIONAL);
	//ADD_INFO(DCMIDiscovery::CapabilitySumaryInfo, "Capabilities Summary");


	//Check basic command
	ADD_PACKAGE_PARAM(DCMIBasics, getCurrentTransport());

	//Check for CipherSuites supported in Platform
	ADD_PACKAGE_PARAM(CipherTest, s_MgmtAttrib.PrimChannelNum, getCurrentTransport());

	//Check SEL Suites one by one against the Platform
	ADD_PACKAGE_PARAM(SELTest, getCurrentTransport());

	ADD_PACKAGE_PARAM(SensorTest, getCurrentTransport());

	ADD_PACKAGE_PARAM(SDRTest, getCurrentTransport());

	ADD_PACKAGE_PARAM(ChassisTest, getCurrentTransport());

	//Verify the LAN default configuration and MultiSession support on primary Channel
	ADD_PACKAGE_PARAM(VLanTest, s_MgmtAttrib.PrimChannelNum, getCurrentTransport());

	ADD_PACKAGE_PARAM(SOLTest, s_MgmtAttrib.PrimChannelNum, getCurrentTransport());

	//Check for TMode Support
	ADD_PACKAGE_PARAM(TmodeTest, s_DCMICapab.smgmt_capab.bOOBTmodePresent, s_MgmtAttrib.TMODEChannelNum, getCurrentTransport());

	ADD_PACKAGE_PARAM(PwrMgmtTest, s_DCMICapab.sopt_capab.bPowerMgmt, getOptPlatformAttrib(), getCurrentTransport());

	ADD_PACKAGE_PARAM(LanTest, s_MgmtAttrib.PrimChannelNum, getCurrentTransport());
}


DCMICipherCheck::DCMICipherCheck(int ver) :
		DCMIDiscovery(ver, TransportManager::getCurrentTransport())
{
	s_Mgmt_Access_Attrib &s_MgmtAttrib = getMgmtAccessAttrib();

	//Check for CipherSuites supported in Platform
	ADD_PACKAGE_PARAM(CheckCiphersTest, s_MgmtAttrib.PrimChannelNum, getCurrentTransport());
}


}
