/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2011 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/

#include "parameters.h"
#include "DCMIClasses.hpp"

using TransportPlugin::Logger;

namespace DCMIClasses
{

static void _DCMIExtId_Info(std::ios_base &ios, IpmiDCMIResponse &response)
{
	std::ostream &stream = dynamic_cast<std::ostream &>(ios);
	stream<< "\n\tThe DCMI Discovery Group Extension code of "
			<< std::hex << std::showbase
			<< static_cast<int>(response.getDCMIExtId())
			<< "\tis "<< (response.isDCMIExtId()?"correct.":"incorrect.");
}

//std::_Smanip<IpmiDCMIResponse &> DCMIExtIdInfo(IpmiDCMIResponse &response)
//{
//   return (std::_Smanip<IpmiDCMIResponse &>(&_DCMIExtId_Info, response));
//}

/**
 * Print info
 *
 * @param stream - stream to print out data
 * @param ob - object to print out
 *
 * @return stream
 */
/*
void IpmiDCMIResponse::print(std::ostream & stream)
{
	print_header(stream);
	stream << DCMIExtIdInfo(*this);

	if (getLength()>1)
	{
		stream << "\n\tData: ";
		std::copy(&at(1), &at(getLength()), std::ostream_iterator<int>(stream, ","));
	}
}
*/

DCMIBaseTest::DCMIBaseTest(std::string name, TransportPlugin::TransportPlugIn &transport) :
		TestSuite(name),
		m_Transport(transport)
{}


void
DCMIBaseTest::sendIpmiCmd(CommonClasses::Request &req, CommonClasses::Response &resp, LogLevel level, byte_t retryCount)
{
    resp.clear(); //clear response
    Logger()(static_cast<unsigned int>(level)) <<"\n\t"<< req ;

	switch ( m_Transport.sendIpmiCmd(req, resp, retryCount) )
	{
		case E_NETWORK_PROBLEM: {
			//Error associated with command completion due to network / packet loss etc
			LOG_WARN<<"\n\tCommand could not be issued due to network error(s)";
			throw TestUtil::TR_NETWORK_ERR;
		}
		default:
		case E_OTHER: {
			LOG_WARN<<"\n\tCommand could not be issued due unknown error ";
			throw TestUtil::TR_FAILURE;
		}
		case E_OK: {
			Logger()(static_cast<unsigned int>(level))<<resp;
			if (!resp.isValidResponse())
			{ //Response is invalid ...
				if (!resp.isCompletionCodeValid())
				{ //Error associated with command completion FW problem ...etc
					LOG_WARN << "\n\tCompletion code ("
						<<std::hex<<std::showbase<<static_cast<int>(resp.getCompCode());
					LOG_WARN	<<") " <<CommonClasses::Response::getCompletionCodeDescryption(resp.getCompCode()) ;
					throw TestUtil::TR_CC_FAILURE;
				}
				if (!resp.isValidSize())
				{ //Error associated with command completion due to network / packet loss etc
					LOG_WARN << "\n\tReceived invalid response - Data length is not compliant";
					throw TestUtil::TR_INVALID_LEN;
				}
				LOG_WARN << "\n\tReceived invalid response for Issued Command.";
				throw TestUtil::TR_FAILURE;
			}//if
		}
	}//switch
}

//set Privilege level for OOB
int
DCMIBaseTest::setSessionPrivilegeLevel(PrivelegeLevel level) throw()
{
	//Privilege level needs to be set only for OOB as it is not applicable - for inband.
	if (m_Transport.connectionType() == TP_INB)
	{
		return TestUtil::TR_SUCCESS;
	}

	//Build Command Object
	CommonClasses::IpmiRequest  request(APPLICATION, 0x3B, level);
	CommonClasses::IpmiResponse response(SET_PRIV_MIN_RSPLEN, SET_PRIV_MAX_RSPLEN);

	//Debug purposes only
	LOG_DEBUG<<"\n\tSetting Session Privilege Level";
	try
	{ //Send request data to the transport
		sendIpmiCmd(request, response, TransportPlugin::logLevelDebug);
		LOG_DEBUG<<"\n\tPrivilege Level set to ";
		switch(level)
		{
		case PRIV_USER:
			LOG_DEBUG<<"USER";
			break;
		case PRIV_OPERATOR:
			LOG_DEBUG<<"OPERATOR";
			break;
		case PRIV_ADMIN:
			LOG_DEBUG<<"ADMIN";
			break;
		}
	}
	catch(TestUtil::TestResult &e) {
	    LOG_ERR<<"\n\tFailed to set Session Privilege Level to the UUT.";
		return e;
	}
	catch(...) {
		LOG_ERR<<"\n\tFailed to set Session Privilege Level to the UUT.";
		return TestUtil::TR_FAILURE;
	}

	return TestUtil::TR_SUCCESS;
}

/*************************************************************************
Method  declaration
DeviceIdTest( .. );
Function Name   :	DeviceIdTest
Purpose         :	Issues and processes the Get Device ID command.
Parameters      :  [IN] any_value - not used .. but necessary for test framework

Returns         :   command completion status.
					This is the value returned from the transport layer, and reports
					success or failure while transferring the command across the transport
					(rmcp+, Kcs etc). The possible return values please @see TestResult
					as well as Exception are welcome

***************************************************************************/
TestResult
DCMIBaseTest::DeviceIdTest(any_value &)
{
	LOG_INFO<<"\nIssue Get Device ID command.";

	CommonClasses::IpmiRequest  request(APPLICATION, 0x01);
	CommonClasses::IpmiResponse response(GET_DEVID_MIN_RSPLEN, GET_DEVID_MAX_RSPLEN);

	setSessionPrivilegeLevel(PRIV_USER);

	//Send request data to the transport
	sendIpmiCmd(request, response);

	//If valid completion code display FW revision to user.
	LOG_INFO<<"\n\tManagement Controller Firmware Major Revision  :\t"<< std::hex<<std::showbase<<static_cast<int>(response[2]);
	LOG_INFO<<"\n\tManagement Controller Firmware Minor Revision  :\t"<< std::hex<<std::showbase<<static_cast<int>(response[3]);

	return TestUtil::TR_SUCCESS;
}

}//namespace DCMIClasses
