/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intel�s	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*******************************************************************************
***		 File		:	DCMI_PwrMgmt_lib.cpp       							  **
***																			  **
***		 Abstract	:	Implements the PowerManagement Testcases              **
***                     in the DCMI_Conformance Module.                      **
***						Also, implements the low level functions			  **
***						that send the "PowerManagememt" commands			  **
***						in Table 6.1Platform Command Definition of DCMI spec  **
***																		      **
***		 Contents	:														  **
***				Definition of following PwrMgmntTest Class					  **
********************************************************************************/

#include "parameters.h"
#include "DCMI_PwrMgmt_lib.h"

using std::hex;
using std::uppercase;

//Definitions for Power Management library functions
#define POWER_STATISTICS 0x01
//Define the BMC Address
#define BMC_ADD  0x20
//Define the requestor LUN, normally 0x00
#define LUN  0x00

const static byte_t PWRMGMT_CC [] = {0x00, 0x80};

/********************************************************************************
Function Name: getPowerReading
Purpose:	  This command is used to get the power reading of the Server
			  And sends the IPMI command sequence to the designated interface to
			  perform the selected action on the UUT. It copies the response to
			  the s_getPwrReading structure which is an OUT Param
Returns    :  s_getPwrReading structure
***************************************************************************/
PwrMgmtTest::s_getPwrReading
PwrMgmtTest::getPowerReading()
{
	DCMIClasses::IpmiDCMIRequest request(0x02);
	request.append<byte_t>(POWER_STATISTICS)/// Mode - 01 (Power Statistics)
			.append<byte_t>(0x00)
			.append<byte_t>(0x00);

	//Build Command Object
	DCMIClasses::IpmiDCMIResponse response(GET_PWR_READING_MIN_RSPLEN, GET_PWR_READING_MAX_RSPLEN);

	//Min Prev Level for GetLANConfig command is  : Operator
	setSessionPrivilegeLevel(PRIV_OPERATOR);

	//Send request data to the transport
	sendIpmiCmd(request, response);

	s_getPwrReading s_PwrReading = {0};
	//Return Command Completion Status

	s_PwrReading.CurPwr = ((int(response[2])<<8) + int(response[1]));
	s_PwrReading.MinPwr = ((int(response[4])<<8) + int(response[3]));
	s_PwrReading.MaxPwr = ((int(response[6])<<8) + int(response[5]));
	s_PwrReading.AvgPwr = ((int(response[8])<<8) + int(response[7]));
	s_PwrReading.IPMITimeStamp = ((UINT32(response[12])<<24) +(UINT32(response[11])<<16) +(UINT32(response[10])<<8) + UINT32(response[9]));
	s_PwrReading.StatTimePeriod = ((UINT32(response[16])<<24) +(UINT32(response[15])<<16) +(UINT32(response[14])<<8) + UINT32(response[13]));
	s_PwrReading.PwrRdngState = int(response[17]);

	return s_PwrReading;
}


/*****************************************************************************
Function Name:	getPowerLimit
Purpose:	This command is used to get the power Limit of the Server
			And sends the IPMI command sequence to the designated interface to
			perform the selected action on the UUT.

Returns  : s_getPwrLimit structure,

Errors are reported by exception
*************************************************************************************/
PwrMgmtTest::s_getPwrLimit
PwrMgmtTest::getPowerLimit()
{
	DCMIClasses::IpmiDCMIRequest request(0x03);
	request.append<byte_t>(0x00)
		   .append<byte_t>(0x00);

//    //Check rsAddr and rsLun usage
//    //reqData.rsAddr = gsOptAttrib.uPMDevSlaveAddr ;
//    request.setRsAddr(gsOptAttrib.uPMDevSlaveAddr);
//
//    //reqData.rsLun = ((gsOptAttrib.uPMChannelNum & 0xF0)>>4);
//    request.setRsLun(((gsOptAttrib.uPMChannelNum & 0xF0)>>4));

	//Build Command Object
	DCMIClasses::IpmiDCMIResponse response(GET_PWR_LIMIT_MIN_RSPLEN, GET_PWR_LIMIT_MAX_RSPLEN, PWRMGMT_CC, ARRAYSIZE(PWRMGMT_CC));

	//Min Prev Level for GetLANConfig command is  : Operator
	setSessionPrivilegeLevel(PRIV_OPERATOR);

	//Send request data to the transport
	//comp code 0x00, 0x80 are valid .... well OEM codess to but now are not...
	sendIpmiCmd(request, response);

	s_getPwrLimit s_PwrLimit = {0};
	//Return Command Completion Status
	//Copy the response data to the return fields

 	s_PwrLimit.Active = response.getCompCode();
 	if(s_PwrLimit.Active == 0x00)
 	{
 		s_PwrLimit.ExceptionAction = response[3];
 		s_PwrLimit.PowerLimitReq = ((int(response[5])<<8) + int(response[4]));
 		s_PwrLimit.CorrTime = ((UINT32(response[9])<<24) +(UINT32(response[8])<<16) +(UINT32(response[7])<<8) + UINT32(response[6]));
 		s_PwrLimit.SampPeriod = ((int(response[13])<<8) + int(response[12]));
 	}
	return s_PwrLimit;
}

/**************************************************************************
 Function Name:		GetPwrReading
 Purpose:			Implements test case for Get Power Reading Command
				Return pass/fail depending on completion code
**************************************************************************/
TestResult
PwrMgmtTest::GetPwrReadingTest(any_value &)
{
	//Confirm that The Power Management Controller is the Primary BMC
	LOG_DEBUG<<"\nGet Current Power Reading Test";
	if (m_power==0)
	{
		LOG_WARN<<"\nPower Management is NOT Supported and it is Optional.";
		return TestUtil::TR_SKIPPED;
	}

	//Print the PM Controller Info obtained from the DCMI Capabilities info command
	LOG_INFO<<"\n\tThe Power Mgmt Controller Address is           :\t0x"<<hex<< int(s_optattrib.uPMDevSlaveAddr);
	LOG_INFO<<"\n\tThe Power Mgmt Controller Channel num          :\t0x"<<hex<< int((s_optattrib.uPMChannelNum & 0xF0)>>4);
	//if Power Management controller is Satelite controller other than BMC -
	//then skip the commands, as SendMessage feature is not defined well currently
	if( s_optattrib.uPMDevSlaveAddr != BMC_ADD || ((s_optattrib.uPMChannelNum & 0xF0)>>4) != LUN)
	{
		LOG_ERR<<"\n\tTo be implemented IPMI bridge mechanism for Satellite Controller....\n";
		return TestUtil::TR_FAILURE;
	}
	//Issue Get Power Reading command and obtain the PowerReading
	s_getPwrReading sPwrReading = getPowerReading();

	//Print and analyse the response obtained from Get Power Reading
	LOG_INFO << "\n\tDCMI Get Power Reading supported";
	//Print Current Power in Watts, and other information from response
	LOG_INFO<<"\n\tSystem Current Power in Watts           : "<<uppercase<<(sPwrReading.CurPwr);
	LOG_INFO<<"\n\tMinimum over sampling duration in Watts : "<<uppercase<<sPwrReading.MinPwr;
	LOG_INFO<<"\n\tMaximum over sampling duration in Watts : "<<uppercase<<(sPwrReading.MaxPwr);
	LOG_INFO<<"\n\tAverage over sampling duration in Watts : "<<uppercase<<(sPwrReading.AvgPwr);

	//check status of Power Limit - bit 6
	switch (sPwrReading.PwrRdngState & 0x40)
	{
	case 0:
		LOG_INFO<<"\n\tPower Measurement not available.";
		break;
	case 0x40:
		LOG_INFO<<"\n\tPower Measurement is active. ";
		break;
	default:
		LOG_DEBUG<<"\n\tInvalid Power Limit Status in response for Get Power Reading from UUT.";
	}
	return TestUtil::TR_SUCCESS;
}

/**************************************************************************
 Function Name:		GetPwrLimit
 Purpose:			Implements test case for Get Power Limit Command
					Get Current Power Limit.
					Return pass/fail depending on completion code
**************************************************************************/
TestResult
PwrMgmtTest::GetPwrLimitTest(any_value &)
{
	LOG_DEBUG<<"\nGet Current Power Limit test";
	if (m_power==0)
	{
		LOG_WARN<<"\nPower Management is NOT Supported and it is Optional.";
		return TestUtil::TR_SKIPPED;
	}

	//Print the PM Controller Info obtained from the DCMI Capabilities info command
	LOG_INFO<<"\n\tThe Power Mgmt Controller Address is           :\t0x"<<hex<< int(s_optattrib.uPMDevSlaveAddr);
	LOG_INFO<<"\n\tThe Power Mgmt Controller Channel num          :\t0x"<<hex<< int((s_optattrib.uPMChannelNum & 0xF0)>>4);
	//if Power Management controller is Satelite controller other than BMC -
	//then skip the commands, as SendMessage feature is not defined well currently
	if( s_optattrib.uPMDevSlaveAddr != BMC_ADD || ((s_optattrib.uPMChannelNum & 0xF0)>>4) != LUN)
	{
		LOG_ERR<<"\n\tTo be implemented IPMI bridge mechanism for Satellite Controller....\n";
		return TestUtil::TR_FAILURE;
	}

	//Issue Get PowerLimit and obtain the response
	s_getPwrLimit s_PwrLimit = getPowerLimit();
	switch( s_PwrLimit.Active )
	{
	case 0x00: {
		LOG_INFO<<"\n\tSystem Power Limit Active";
		//Print System Power Limit in Watts, and other information from response
		if(s_PwrLimit.ExceptionAction==0x01)
			LOG_INFO<<"\n\tException Action                             : Power Off";
		else
			if(s_PwrLimit.ExceptionAction==0x11)
				LOG_INFO<<"\n\tException Action                             : SEL Logging";
			else
				LOG_INFO<<"\n\tException Action                             : Unknown";
		LOG_INFO<<"\n\tSystem  Power Limit in Watts                 : "<< s_PwrLimit.PowerLimitReq;
		LOG_INFO<<"\n\tSystem  Correction time limit in milliseconds: "<< s_PwrLimit.CorrTime;
		LOG_INFO<<"\n\tStatistics Sampling period in seconds        : "<< s_PwrLimit.SampPeriod;
		}
		break;

	case 0x80:
		LOG_INFO<<"\n\tSystem Power Limit is not Active ";
		break;

	default:
		LOG_INFO<<"\n\tSystem Power Limit is unknown (OEM defined)";
	};

	return TestUtil::TR_SUCCESS;
}

