/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intel�s	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/********************************************************************************
***		 File		:	DCMI_SEL_lib.cpp									  **
***																			  **
***		 Abstract	:	Implements all the low level procedures for the       **
***				        'Logging Commands' - Get SEL Info, Reserve SEL etc    **
***                     in Table 6-1 Platform Command Definition of           **
***						DCMI spec.                                            **
***																		      **
***		 Contents	:														  **
***				Definition of following SelTest class                         **
***			                    											  **
***				Definition of following local functions                       **
***				GetSelInfo                              					  **
***				GetSelEntry     											  **
***				ReserveSel         									          **
***				ClearSel                									  **
***				GetSelEntryTest  											  **
***				GetLastSelEntryTest      									  **
***				GetFirstSelEntryTest      									  **
***				ReserveSelTest     									          **
***				ClearSelTest               									  **
***				VerifyClearSelTest         									  **
***				GetSelInfoTest												  **
********************************************************************************/
#include "parameters.h"
#include "declarations.h"
#include "DCMI_Configuration.h"
#include "DCMI_SEL_lib.h"

using std::hex;
using std::uppercase;

/*************************************************************************
Function declaration
void DisplaySelValue(struct s_sel_entry)
Function Name:		DisplaySelValue
Purpose:			Displays the SEL Entry passed as parameter
Parameters:			[IN] sSelData - The structure with SEL information
									to be displayed
Returns:		    none
*************************************************************************/
std::ostream &operator<<(std::ostream &stream, struct s_sel_entry &sSelData)
{
	stream<<"\n\tSEL Values are:";
	stream<<"\n\t\tRecord ID:\t 0x"<<hex<<uppercase<< (int)sSelData.iRecID;
	stream<<"\n\t\tNext Record ID:\t 0x"<< hex<<uppercase<< (int)sSelData.iNextRecID;
	stream<<"\n\t\tRecord Type:\t 0x"<<hex<< uppercase<< (int)sSelData.bRecType;
	stream<<"\n\t\tTimestamp:\t 0x"<<hex<< uppercase<< (int)sSelData.iTimeStamp;
	stream<<"\n\t\tGenerator ID:\t 0x"<< hex<<uppercase<< (int)sSelData.iGenID;
	stream<<"\n\t\tEvent Msg Rev:\t 0x"<<hex<< uppercase<< (int)sSelData.bEventMsgRev;
	stream<<"\n\t\tSensor Type:\t 0x"<< hex<<uppercase<<(int)sSelData.bSensorType;
	stream<<"\n\t\tSensor Number:\t 0x"<<hex<< uppercase<<(int)sSelData.bSensorNum;
	stream<<"\n\t\tEvent Type:\t 0x"<<hex<< uppercase<<(int)sSelData.bEventType;
	stream<<"\n\t\tEvent Data 1:\t 0x"<<hex<< uppercase<<(int)sSelData.bEventData1;
	stream<<"\n\t\tEvent Data 2:\t 0x"<<hex<< uppercase<<(int)sSelData.bEventData2;
	stream<<"\n\t\tEvent Data 3:\t 0x"<<hex<< uppercase<<(int)sSelData.bEventData3;

	return stream;
}

/*********************************************************************************
Function Declaration
int GetSelInfo()

Purpose  : Tests the Get SEL Info Command

Return   : SELentries - The SEL entries obtained

***********************************************************************************/
int
SELTest::getSelInfo()
{
   //Build Command Object
   CommonClasses::IpmiRequest request(STORAGE, 0x40);
   CommonClasses::IpmiResponse response(GET_SEL_INFO_MIN_RSPLEN, GET_SEL_INFO_MAX_RSPLEN);

   //Send request data to the transport
   //Process return Data  & Verify Completion Code, errors are reported by exception
   sendIpmiCmd(request, response);

   //Get SEL Values
   int SELentries = response[1] + (static_cast<int>(response[2]) << 8);

   LOG_INFO<<"\n\tNumber of SEL entries is: 0x"<<hex<< SELentries;
   return SELentries;
}

TestResult
SELTest::GetSelInfoTest(any_value &)
{
   LOG_DEBUG<<"\nGet SEL Info Test.";
   setSessionPrivilegeLevel(PRIV_USER);
   m_SelEntries = getSelInfo();

   return TestUtil::TR_SUCCESS;
}

/*****************************************************************************
Function Name:	ReserveSelTest

Purpose:		Reserves the SEL for read write operations and ensures the
				command is supported

Returns:		0 - Test Passed
				1 - Test Failed (or)
				TRANSPORT_ERROR -Network Error
******************************************************************************/
//int
TestResult
SELTest::ReserveSelTest(any_value &)
{
	LOG_DEBUG<<"\nIssue the Reserve SEL command.";
	m_ResId = getReserveSel();
	//wait for a while and allow system time to "get over"
	sleep_printdots(m_TimeOutSELReserve);

	return TestUtil::TR_SUCCESS;
}

SELTest::reserv_id
SELTest::getReserveSel()
{
	//Build Command Object
	CommonClasses::IpmiRequest request(STORAGE, 0x42);
	CommonClasses::IpmiResponse response(RESERVE_SEL_MIN_RSPLEN, RESERVE_SEL_MAX_RSPLEN);

	//Issue Transport Request Function with Reserve SEL Info operation parameters
	//Send request data to the transport
	sendIpmiCmd(request, response);
	//Get/Store Reservation ID
	reserv_id resid;

	resid.ridLsb = (response[0]);
	resid.ridMsb = (response[1]);

	return resid;
}

/*************************************************************************
Function declaration
Function Name:		getSelEntry
Purpose:			Gets specific SEL Record.

Parameters:			[IN] reservationIdLsb, reservationIdMsb -
						Reservation ID required for the Request Packet to get Sel record
					[IN] recordIdLsb, recordIdMsb - The recordID in SEL to be read

					[OUT] - s_sel_entry - The SEL record
 ***************************************************************************/
s_sel_entry
SELTest::getSelEntry(reserv_id resid, record_id recid)
{
	//Build Command Object
	CommonClasses::IpmiRequest request(STORAGE, 0x43);

	request.append<byte_t>(resid.ridLsb)
   				   .append<byte_t>(resid.ridMsb)
   				   .append<byte_t>(recid.rLsb)
   				   .append<byte_t>(recid.rMsb)
   				   .append<byte_t>(0x00)
   				   .append<byte_t>(0x10);

	CommonClasses::IpmiResponse response(GET_SEL_ENTRY_MIN_RSPLEN, GET_SEL_ENTRY_MAX_RSPLEN);

	//Issue Transport Request Function with  SEL entry operation parameters
	//Send request data to the transport
	sendIpmiCmd(request, response);

	/*
	 * Command executed successfully and Completion code is 0 - copy the SEL entry
	 */
	s_sel_entry s_SelEntry = response.getFromData<s_sel_entry>(0);
	return s_SelEntry;
}

/***************************************************************************
Function Name:	GetSelEntryTest
Purpose:		Gets specific SEL Record by calling GetSelEntry(),
				and check if valid response

Parameters	:	[IN] Reservation ID required for the Request Packet to get Sel record
******************************************************************************/

TestResult
SELTest::GetSelEntryTest(any_value &an_rid)
{
	LOG_DEBUG<<"\nGets specific SEL Record test";
	if(m_SelEntries==0)
	{
		//Number of SEL entries is 0, skipping the test
		LOG_WARN<<"\n\tWarning: There are no SEL entries, skipping the test";
		return TestUtil::TR_SKIPPED;
	}

	record_id recid(0,0);
	if(!an_rid.empty())
	{
		recid = an_rid.value<record_id>();
	}

	reserv_id resid = {0} ;
	s_sel_entry s_SelEntry = getSelEntry(resid, recid) ;
	//Get SEL Values and display to screen
	LOG_INFO << s_SelEntry;
	//wait for a while and allow system time to "get over"
	sleep_printdots(m_TimeOutSELGet);

   return TestUtil::TR_SUCCESS;
}


TestResult
SELTest::GetReservSelEntryTest(any_value &an_rid)
{
	LOG_DEBUG<<"\n\tIssue Get reserve SEL .";
	if(m_SelEntries==0)
	{
		//Number of SEL entries is 0, skipping the test
		LOG_WARN<<"\n\tThere are no SEL entries, skipping the test";
		return TestUtil::TR_SKIPPED;
	}

	record_id recid(0,0) ;
	if(!an_rid.empty())
	{
		recid = an_rid.value<struct record_id>();
	}

	s_sel_entry s_SelEntry =  getSelEntry(m_ResId, recid) ;
	//Get SEL Values and display to screen
	LOG_INFO << s_SelEntry;
	//wait for a while and allow system time to "get over"
	sleep_printdots(m_TimeOutSELGet);

	return TestUtil::TR_SUCCESS;
}


void
SELTest::clearSel(reserv_id resid, byte_t clrAction, std::vector<byte_t> &rsp_dest)
{
	//Build Command Object
	CommonClasses::IpmiRequest request(STORAGE, 0x47);

	request .append<byte_t>(resid.ridLsb)
			.append<byte_t>(resid.ridMsb)
			.append<byte_t>(0x43)
			.append<byte_t>(0x4C)
			.append<byte_t>(0x52)
			.append<byte_t>(clrAction);

	CommonClasses::IpmiResponse response(CLEAR_SEL_MIN_RSPLEN, CLEAR_SEL_MAX_RSPLEN);

	//Issue Transport Request Function with  SEL entry operation parameters
	//Send request data to the transport
	sendIpmiCmd(request, response);

	response.copy_to(0, rsp_dest);
}


/*****************************************************************************************
Function Name:	ClearSel
Purpose		:	Clears the SEL NVRam Area.
********************************************************************************************/
TestResult
SELTest::ClearSelTest(any_value &)
//(int reservationIdLsb, int reservationIdMsb)
{
	LOG_DEBUG<<"\nReserve SEL and prepare for SEL Clear.";
	reserv_id resid = getReserveSel();

	//wait for a while and allow system time to "get over"
	sleep_printdots(m_TimeOutSELReserve);

	LOG_DEBUG<<"\n\tIssue SEL Clear";

	//Minimum Privilege level for Clear SEL Command: Operator
	setSessionPrivilegeLevel(PRIV_OPERATOR);

	std::vector<byte_t> response;

	//Issue Transport Request Function with  SEL entry operation parameters
	//Send request data to the transport
	clearSel(resid, 0xAA, response);
	//wait for a while and allow system time to "get over"
	sleep_printdots(m_TimeOutSELClear); // Was 1sek delay??

	return TestUtil::TR_SUCCESS;
}


/*************************************************************************
Function Name:	VerifyClearSelTest
Purpose:		Checks if the previous Clear SEL NVRAM area was successful
************************************************************************/
TestResult
SELTest::VerifyClearSelTest(any_value &)
//(int reservationIdLsb, int reservationIdMsb)
{
	LOG_DEBUG<<"\nVerify SEL Clear action.";
	reserv_id resid = getReserveSel();
	//wait for a while and allow system time to "get over"
	sleep_printdots(m_TimeOutSELReserve);

	LOG_DEBUG<<"\n\tIssue SEL Clear Action Verify.";

	std::vector<byte_t> response;
	//Issue Transport Request Function with  SEL entry operation parameters
	//Send request data to the transport
	clearSel(resid, 0x00, response) ;
	//wait for a while and allow system time to "get over"
	sleep_printdots(m_TimeOutSELClear);

	//Check if action set to 0x00 that return byte checks for SEL Cleared.
	if (response[0] == 0x01)
	{
		LOG_INFO<<"\n\tSEL is indicated as cleared.";
		return TestUtil::TR_SUCCESS;
	}

	LOG_INFO<<"\n\tSEL is indicated as NOT Cleared.";
	return TestUtil::TR_FAILURE;
}


//*************************************************************************
void SELTest::doConfigure(DcmiConfiguration &option)
{
	strtotimeout(option.getConfigurationValue("TimeOutValues","TimeOutSELReserve"), m_TimeOutSELReserve);
	strtotimeout(option.getConfigurationValue("TimeOutValues","TimeOutSELGet"), m_TimeOutSELGet);
	strtotimeout(option.getConfigurationValue("TimeOutValues","m_TimeOutSELClear"), m_TimeOutSELClear);
};
































