/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2011 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*
 *
 * DCMI_Thermal_lib.cpp
 *
 *  Created on: 10-03-2011
 */

#include "parameters.h"
#include "DCMI_Configuration.h"
#include "DCMI_Thermal_lib.h"

void ThermalTest::themperatureReading(byte_t entity_id, byte_t entity_nr)
{
	LOG_DEBUG << "\nGet Temperature Reading (" << std::hex
			<< std::showbase << static_cast<int> (entity_id) << ")";

	//Build Command Object
	DCMIClasses::IpmiDCMIRequest request(0x10);
	request.append<byte_t>(entity_nr)
		   .append<byte_t>(entity_id)
		   .append<byte_t>(0x00)
		   .append<byte_t>(0x00);

	DCMIClasses::IpmiDCMIResponse response(GET_THERMAL_RDNG_MIN_RSPLEN, GET_THERMAL_RDNG_MAX_RSPLEN);

	//Send request data to the transport
	sendIpmiCmd(request, response);

	//number of available instances
	byte_t value = response.getFromData<byte_t> (2);
	LOG_INFO << "\n\tAvailable instances for " << std::hex
			<< std::showbase << static_cast<int>(entity_id) << ": "
			<< std::dec << static_cast<int>(value);

	//loop insides real sets of temperature Data
	struct temperature_data data;
	for (int i = 0; i < value; i++)
	{
		data = response.getFromData<struct temperature_data>(3 + (i * sizeof(struct temperature_data)));
		LOG_INFO << "\n\t\tSensor number: " << static_cast<int>(data.idx)
							<< " value: " << ((data.sign) ? '-' : '+')
							<< static_cast<int>(data.value) << "C";
	}
}

ThermalTest::thermal_info
ThermalTest::getThermalLimit(byte_t entity_id, byte_t entity_nr)
{
	LOG_DEBUG << "\nGet Thermal limit (" << std::hex << std::showbase
			    << static_cast<int>(entity_id) << ")";

	DCMIClasses::IpmiDCMIRequest request(0x0C);

	request.append<byte_t>(entity_id)
		   .append<byte_t>(entity_nr);

	DCMIClasses::IpmiDCMIResponse response(GET_THERMAL_MIN_RSPLEN, GET_THERMAL_MAX_RSPLEN);

	//Send request data to the transport
	sendIpmiCmd(request, response);//, GET_THERMAL_MIN_RSPLEN, GET_THERMAL_MAX_RSPLEN);

		//Exception value ..
	thermal_info info = {0};

	info.action = response.getFromData<exception_action> (1);

	LOG_INFO << "\n\tException Action code: " << std::hex << std::showbase
			<< static_cast<int>(*reinterpret_cast<byte_t*> (&(info.action)))
			<< std::endl
			<< (info.action.poweroff ? "\t\t-Hard Power Off system and log event\n" : "")
			<< (info.action.log ? "\t\t-Log event to SEL only"	: "") << std::endl;

	//Temperature Limit ..
	info.limit = response.getFromData<byte_t> (2);
	LOG_INFO << "\tTemperature Limit: " << static_cast<int>(info.limit)
						<< ", hex: " << std::hex << std::showbase
						<< static_cast<int>(info.limit) << std::endl;

	//Exception time ..
	info.exTime = response.getFromData<short>(3);
	LOG_INFO << "\tException Time interval in seconds: "	<< static_cast<int>(info.exTime);

	return info; //pas
}

void
ThermalTest::setThermalLimit(byte_t entity_id, byte_t entity_nr)
{
	LOG_DEBUG << "\nSet Thermal limit (" <<std::hex<<std::showbase<< static_cast<int>(entity_id) << ")";

	DCMIClasses::IpmiDCMIRequest request(0x0B);

	request.append<byte_t>(entity_id)
		   .append<byte_t>(entity_nr);

	exception_action action = { 0 };

	action.log = 1;

	//request data: Action.Limit.Time
	request.append<byte_t>(*reinterpret_cast<byte_t*>(&action))
		   .append<byte_t>(temp_limit)
		   .append<short>(exception_time);

	DCMIClasses::IpmiDCMIResponse response(SET_THERMAL_MIN_RSPLEN, SET_THERMAL_MAX_RSPLEN);

	LOG_INFO << "\n\tException Action code: " <<std::hex<<std::showbase<< static_cast<int>(*reinterpret_cast<byte_t*> (&action))<< std::endl
			<< (action.poweroff ? "\t\t-Hard Power Off system and log event\n": "")
			<< (action.log ? "\t\t-Log event to SEL only" : "")
			<< "\n\tTemperature Limit: "<<std::dec<< static_cast<int>(temp_limit) << ", hex: " <<std::hex<<std::showbase<< static_cast<int>(temp_limit)
			<< "\n\tException Time interval in seconds: "<<std::dec<<static_cast<int>(exception_time);

	//Send request data to the transport
	sendIpmiCmd(request, response);//, SET_THERMAL_MIN_RSPLEN, SET_THERMAL_MAX_RSPLEN);
	//passs
}

