/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2011 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*
 * DCMI_Wdt_lib.cpp
 *
 *  Created on: 02-05-2011
 *
 */

#include "parameters.h"
#include "DCMI_Configuration.h"
#include "DCMI_Wdt_lib.h"

const static byte_t WDT_CC [] = {0x00, 0x80};

void
WatchdogTimerTest::resetWatchdogTimer()
{
	CommonClasses::IpmiRequest request(APPLICATION, 0x22);
	CommonClasses::IpmiResponse response(0, 0, WDT_CC, ARRAYSIZE(WDT_CC)); // 0, 0
	sendIpmiCmd(request, response);
	if (response.getCompCode() == 0x80)
	{
		LOG_DEBUG << "Attempt to start un-initialized watchdog!";
	}
}


WatchdogTimerTest::wdt_timer
WatchdogTimerTest::getWatchdogTimer()
{
	CommonClasses::IpmiRequest request(APPLICATION, 0x25);
	CommonClasses::IpmiResponse response(GET_WDT_MIN_RSPLEN, GET_WDT_MAX_RSPLEN);
	sendIpmiCmd(request, response);

	wdt_timer timer = response.getFromData<wdt_timer> (0);

	LOG_DEBUG << "\n\tTimer Use code: " << std::hex << std::showbase
			<< static_cast<int>(*reinterpret_cast<byte_t*> (&(timer.use)));
	LOG_DEBUG << "\n\tTimer Action code: " << std::hex << std::showbase
			<< static_cast<int>(*reinterpret_cast<byte_t*> (&(timer.action)));
	LOG_DEBUG << "\n\tTimer initial pre-timeout value: " << std::hex << std::showbase
			<< static_cast<int>(timer.pre_timout);
	LOG_DEBUG << "\n\tTimer Expiration code: " << std::hex << std::showbase
			<< static_cast<int>(*reinterpret_cast<byte_t*> (&(timer.expiration)));
	LOG_DEBUG << "\n\tTimer initial countdown value: " << std::hex << std::showbase
			<< static_cast<int>(*reinterpret_cast<short*> (&(timer.init_countdown_lsb)));
	LOG_DEBUG << "\n\tTimer present countdown value: " << std::hex << std::showbase
				<< static_cast<int>(*reinterpret_cast<short*> (&(timer.present_countdown_lsb)));

	return timer; //pas
}
