/*******************************************************************************
********************************************************************************
***                                                                           **
***	                           INTEL CONFIDENTIAL                             **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***	                           All Rights Reserved                            **
***                                                                           **
***                 INTEL CORPORATION PROPRIETARY INFORMATION                 **
***                                                                           **
***		The source code contained or described herein and all documents	      **
***		related to the source code ("Material") are owned by Intel            **
***		Corporation or its suppliers or licensors. Title to the Material      **
***		remains with Intel Corporation or its suppliers and licensors.        **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.                  **
***		The Material is protected by worldwide copyright and trade secret     **
***		laws and treaty provisions. No part of the Material may be used,      **
***		copied, reproduced, modified, published, uploaded, posted,	          **
***		transmitted, distributed, or disclosed in any way without Intels      **
***		prior express written permission.                                     **
***	                                                                          **
***		No license under any patent, copyright, trade secret or other         **
***		intellectual property right is granted to or conferred upon you by    **
***		disclosure or delivery of the Materials, either expressly, by         **
***		implication, inducement, estoppel or otherwise. Any license under     **
***		such intellectual property rights must be express and approved by     **
***		Intel in writing.                                                     **
********************************************************************************/


#ifndef _C_EXCEPT_H
#define _C_EXCEPT_H

#ifdef WIN32
#include <windows.h>
#define ERROR_FUNCTION GetLastError()
#else
#include <errno.h>
#define ERROR_FUNCTION errno
#endif

#include <stdexcept>
#include <string>

/**
 * Base class for exceptions thrown on Linux and Windows.
 * The exception uses <code>strerror()</code> to retrieve error
 * description reported by <code>what()</code>.
 */
class CError : public std::runtime_error
{
public:
    /**
     * Ctor.
     *
     * @param func Name of the function that failed.
     * @param err Error code retured by the function <code>func</code>.
     */
    CError(const std::string& func, int err = ERROR_FUNCTION);

    /**
     * Dtor.
     */
    ~CError() throw();

    /**
     * Reports error code and description. The description
     * is retrieved using <code>strerror()</code> function.
     *
     * @return Pointer to the description of exception.
     */
    const char* what() const throw();

    /**
     * Retrieves the error code passed to the ctor.
     *
     * @return Error code passed to the ctor.
     */
    int getErr() const;

private:
    /**
     * Contains the formatted decription returned by <code>what()</code>.
     * The description is formatted on the first <code>what()</code> invocation.
     */
    mutable std::string msg;

    /// Stores the name of the function that failed.
    std::string func;

    /// Stores the error code.
    int err;
};

#endif //#ifndef _C_EXCEPT_H
