/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
#include "Logger.h"

using namespace std;

namespace TransportPlugin
{

Logger::LogCore Logger::logCore;

LogSink::LogSink(int logLevel)
        :logLevel(logLevel)
{
}

LogSink::~LogSink()
{
}

void
LogSink::setLogLevel(unsigned int logLevel)
{
    this->logLevel = logLevel;
}

LogSinkOstream::LogSinkOstream(std::ostream& os, int logLevel)
              :LogSink(logLevel), os(os)
{
}

void
LogSinkOstream::log(unsigned int msgLevel, const char* msg)
{
    if (msgLevel <= this->logLevel)
    {
        os << msg;
        os.flush();
    }
}

Logger::LogCore::LogCore()
{
    pCritSect = OsSpecificCritSect::CreateCritSectObject();
}

Logger::LogCore::~LogCore()
{
    for (size_t i = 0; i < sinks.size(); ++i)
    {
        delete sinks[i];
    }
    delete pCritSect;
}

void
Logger::LogCore::log(unsigned int msgLevel, const char* msg)
{
    pCritSect->EnterCritSect();
    try
    {
        for (size_t i = 0; i < sinks.size(); ++i)
        {
            sinks[i]->log(msgLevel, msg);
        }
    }
    catch (...)
    {};
    pCritSect->LeaveCritSect();
}

Logger::Logger()
{
    if (!logCore.logPrefix.empty())
    {
        *this << logCore.logPrefix << " ";
    }
}

Logger::~Logger()
{
  //  *this;//SM: Removed endl to display Asset Tag in a single line, Didn't wanted endl hardcoded.
    logCore.log(this->msgLevel, str().c_str());
}

void
Logger::setOutputStream(std::ostream& os, unsigned int logLevel)
{
    addLogSink(new LogSinkOstream(os, logLevel));
}

void
Logger::addLogSink(LogSink* logSink)
{
    logCore.sinks.push_back(logSink);
}

void
Logger::setLogPrefix(const char* logPrefix)
{
    stringstream s;
    s << "[" << logPrefix << "]:";
    logCore.logPrefix = s.str();
}

void
Logger::setLogLevel(unsigned int logLevel)
{
    for (size_t i = 0; i < logCore.sinks.size(); ++i)
    {
        logCore.sinks[i]->setLogLevel(logLevel);
    }
}

int
Logger::log(int msgLevel, const char* msg)
{
    try
    {
        logCore.log(msgLevel, msg);
    }
    catch (exception&)
    {
        return 1;
    }
    return 0;
}

}
