/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/

#ifndef _OS_SPECIFIC_CRIT_SECT_H
#define _OS_SPECIFIC_CRIT_SECT_H

namespace TransportPlugin
{

/**
 *
 * Abstract:
 *   Hides OS-dependent implementation of critical section.
 */
class OsSpecificCritSect
{
public:
    /**
     *
     * Destructor.
     * @description
     * Releases resources used by the critical section object. 
     *
     * @returns NONE
     */
    virtual ~OsSpecificCritSect()
    {
    }

    /**
     *
     * Marks the beginning of a critical section.
     * @description
     * Waits for ownership of critical section object.
     *
     * @returns NONE
     */
    virtual void EnterCritSect() = 0;

    /**
     *
     * Marks the end of a critical section.
     * @description
     * Realeases ownership of critical section object.
     *
     * @returns NONE
     */
    virtual void LeaveCritSect() = 0;

    /**
     *
     * Creates a critical section object.
     * @description
     * This static function should be called to create a critical section object. 
     * Implementation of this function is different for each OS. The actual 
     * object that is created belongs to an OS-dependent class which inherits
     * from OsSpecificCritSect.
     *
     * @returns pointer to created critical section object
     */
    static OsSpecificCritSect* CreateCritSectObject();
};

}

#endif // _OS_SPECIFIC_CRIT_SECT_H
