/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/

#ifndef _OS_SPECIFIC_FUNCTIONS_H
#define _OS_SPECIFIC_FUNCTIONS_H

namespace TransportPlugin
{
/**
 **************************************************************
 * Abstract:
 *   Provides wrappers for some OS-specific functions.
 **************************************************************/
class OsSpecificFunctions
{
public:
    /**
     *************************************************************************
     * Compares two strings case insensitively.
     * @description
     * Wrapper for WIN32 _stricmp function and Linux strncmp function.
     *
     * @param str1 - string 1 to compare
     * @param str2 - string 2 to compare
     *
     * @return comparison result
     *************************************************************************/
    static int stricmp(const char* str1, const char* str2);

    /**
     *************************************************************************
     * Copies one string to another.
     * @description
     * Wrapper for WIN32 strcpy_s function and Linux strcpy function.
     *
     * @param dst - destination string
     * @param dstSize - size of destination string
     * @param src - source string
     *
     * @return error code
     *************************************************************************/
    static int strcpys(char* dst, unsigned int dstSize, const char* src);

    /**
     *************************************************************************
     * Formats string.
     * @description
     * Wrapper for WIN32 sprintf_s function and Linux snprintf function.
     *
     * @param dst - destination string
     * @param dstSize - size of destination string
     * @param format - string containing format of the destination string
     *
     * @return number of characters written or error code
     *************************************************************************/
    static int sprintfs(char* dst, unsigned int dstSize, const char* format, ...);
};
}

#endif // _OS_SPECIFIC_FUNCTIONS_H
