/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/

#ifndef _REMOTE_LOGGER_H
#define _REMOTE_LOGGER_H

#include <sstream>
#include <vector>
#include <string>

#include "transport.h"
#include "LogLevel.h"

namespace TransportPlugin
{

/**
 * Logs a single message using configured callback function.
 * Note that setLogPrefix(), setLoggerFunc() are static
 * and configures the logging mechanism, not a single log entry.
 */
class RemoteLogger : public std::ostringstream
{
public:
    /**
     * Ctor
     */
    RemoteLogger();

    /**
     * Dtor that does the message logging job.
     */
    ~RemoteLogger();

    /**
     * Sets the severity of the message being logged by the current instance of the logger.
     * See the LOG_* macros for examples on the log levels that are in use.
     *
     * @param msgLevel Specifies message level of the current log message.
     */
    inline RemoteLogger& operator ()(unsigned int msgLevel);

    /**
     * Configures logging callback function. Each message is logged using the function
     * configured with this method.
     *
     * @param loggerFun Logging function pointer.
     */
    static void setLoggerFunc(dcmi_logger_fun_t loggerFun);

    /**
     * Allows to configure prefix text which is added to each log message.
     *
     * @param logPrefix prefix to be defined.
     */
    static void setLogPrefix(const char* logPrefix);

private:
    /// Log level of the current message.
    unsigned int msgLevel;
    /// Pointer to the function which performs actual logging.
    static dcmi_logger_fun_t pLoggerFun;
    /// Prefix which is added to each log message.
    static std::string logPrefix;
};

inline
RemoteLogger&
RemoteLogger::operator ()(unsigned int msgLevel)
{
    this->msgLevel = msgLevel;
    return *this;
}

}

/// Creates a log message that is logged on error level.
#define LOG_ERR TransportPlugin::RemoteLogger()((unsigned int)TransportPlugin::logLevelErrors)

/// Creates a log message that is logged on warning level.
#define LOG_WARN TransportPlugin::RemoteLogger()((unsigned int)TransportPlugin::logLevelWarnings)

/// Creates a log message that is logged on summary information level.
#define LOG_SUMMARY TransportPlugin::RemoteLogger()((unsigned int)TransportPlugin::logLevelSummary)

/// Creates a log message that is logged on detailed information level.
#define LOG_DETAIL TransportPlugin::RemoteLogger()((unsigned int)TransportPlugin::logLevelDetails)

/// Creates a log message that is logged on warning level.
#define LOG_INFO TransportPlugin::RemoteLogger()((unsigned int)TransportPlugin::logLevelInfo)

/// Creates a log message that is logged on high-level debug information level.
#define LOG_DEBUG TransportPlugin::RemoteLogger()((unsigned int)TransportPlugin::logLevelDebug)

// TBD: the definitions below are for the RMCPP plugin which has not been updated
// to recent changes in the logging functionality yet.
// Remove them.
#define TP_LOG LOG_ERR
#define TP_LOG_V LOG_DEBUG

#endif //_REMOTE_LOGGER_H
