/*++

INTEL CONFIDENTIAL
Copyright 2009 Intel Corporation All Rights Reserved.

The source code contained or described herein and all documents related to the
source code ("Material") are owned by Intel Corporation or its suppliers or
licensors. Title to the Material remains with Intel Corporation or its
suppliers and licensors. The Material contains trade secrets and proprietary
and confidential information of Intel or its suppliers and licensors. The
Material is protected by worldwide copyright and trade secret laws and treaty
provisions. No part of the Material may be used, copied, reproduced, modified,
published, uploaded, posted, transmitted, distributed, or disclosed in any way
without Intel's prior express written permission.

No license under any patent, copyright, trade secret or other intellectual
property right is granted to or conferred upon you by disclosure or delivery
of the Materials, either expressly, by implication, inducement, estoppel or
otherwise. Any license under such intellectual property rights must be express
and approved by Intel in writing.

File Name:
   DCMI_IOCTL.h

Abstract:
   This header file contains all declarations shared between DCMI-HI driver and user applications.

--*/

#ifndef _DCMI_IOCTL_H
#define _DCMI_IOCTL_H

//
// Driver IOCTLs
//
#define FILE_DEVICE_DCMI		0x00008010
#define IOCTL_DCMI_BASE			0x00000880

#define IOCTL_DCMI_SEND_GET_MESSAGE		CTL_CODE(FILE_DEVICE_DCMI, IOCTL_DCMI_BASE + 2, METHOD_BUFFERED, FILE_ANY_ACCESS)


#define INVALID_INPUT_BUFFER             ((NTSTATUS)0xE0070002L)
#define INVALID_OUTPUT_BUFFER            ((NTSTATUS)0xE0070003L)
#define HARDWARE_FAILURE                 ((NTSTATUS)0xE0040008L)
#define DRIVER_FAILURE                   ((NTSTATUS)0xE0040009L)
#define INPUT_BUFFER_TOO_SMALL           ((NTSTATUS)0xE0070011L)
#define INVALID_DRIVER_IOCTL             ((NTSTATUS)0xE007000EL)
#define INVALID_DRIVER_REQUEST           ((NTSTATUS)0xE007000FL)

#define IMB_SEND_TIMEOUT                 ((NTSTATUS)0xE0070004L)
#define IMB_RECEIVE_TIMEOUT              ((NTSTATUS)0xE0070005L)
#define IMB_IF_SEND_TIMEOUT              ((NTSTATUS)0xE0070006L)
#define IMB_IF_RECEIVE_TIMEOUT           ((NTSTATUS)0xE0040007L)
#define IMB_INVALID_IF_RESPONSE          ((NTSTATUS)0xE004000AL)
#define IMB_INVALID_PACKET               ((NTSTATUS)0xE004000BL)
#define IMB_RESPONSE_DATA_OVERFLOW       ((NTSTATUS)0xE004000CL)
#define IMB_INVALID_REQUEST              ((NTSTATUS)0xE007000DL)
#define IMB_CANT_GET_SMS_BUFFER          ((NTSTATUS)0xE0070010L)
#define IMB_MSG_NOT_AVAILABLE            ((NTSTATUS)0xE0070012L)
#define IMB_SEND_ERROR                   ((NTSTATUS)0xE0070013L)

#pragma pack(1)
typedef struct {
	BYTE rsSa;                       // IMB address of responder
	BYTE cmd;                        // IMB command
	BYTE netFn;			 // netFn for command
	BYTE rsLun;			 // logical unit on responder
	BYTE dataLength;	         // length of following data
	BYTE data[1];		         // request data
} ImbRequest;

typedef struct {

   DWORD		flags;	         // request flags
#define				NO_RESPONSE_EXPECTED	0x01	// don't wait around for an IMB response SEnd
   DWORD		timeOut;	 // in uSec units
   ImbRequest	req;			 // message buffer
} ImbRequestBuffer;

typedef struct {
    BYTE       cCode;		         // completion code
    BYTE       data[1];		         // response data buffer
} ImbResponseBuffer;
#pragma pack()

#endif //_DCMI_IOCTL_H

