/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2011 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/

/*
 * DeviceIoControl.c
 *
 *  Created on: 01-06-2011
 */

#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>

#include "DeviceIoControl.h"

/////////////////////////////////////////////////////////////////////////////
// DeviceIoControl
/////////////////////////////////////////////////////////////////////////////
/*F*
//  Name:       DeviceIoControl
//  Purpose:    Simulate NT DeviceIoControl using unix calls and structures.
//  Context:    called for every NT DeviceIoControl
//  Returns:    FALSE for fail and TRUE for success. Same as standarad NTOS call
//              as it also sets Ntstatus.status.
//  Parameters: Standard NT call parameters, see below.
//  Notes:      none
*F*/
BOOL
DeviceIoControl(
	HANDLE 			hDevice, /* handle of device */
	DWORD 			dwIoControlCode, /* control code of operation to perform*/
	LPVOID 			lpvInBuffer, /* address of buffer for input data */
	DWORD 			cbInBuffer, /* size of input buffer */
	LPVOID 			lpvOutBuffer, /* address of output buffer */
	DWORD 			cbOutBuffer, /* size of output buffer */
	LPDWORD 		lpcbBytesReturned, /* address of actual bytes of output */
	LPOVERLAPPED 		lpoOverlapped /* address of overlapped struct */
	)
{
	struct smi s;
	int ioctl_status;
	//IO_STATUS_BLOCK NTstatus; //dummy place holder. See deviceiocontrol.

  	s.lpvInBuffer = lpvInBuffer;
  	s.cbInBuffer = cbInBuffer;
  	s.lpvOutBuffer = lpvOutBuffer;
  	s.cbOutBuffer = cbOutBuffer;
  	s.lpcbBytesReturned = lpcbBytesReturned;
  	s.lpoOverlapped = lpoOverlapped;
//  	s.ntstatus = (LPVOID)&NTstatus; //dummy place holder. Linux IMB driver
//									//doesn't return status or info via it.

	if ( (ioctl_status = ioctl(hDevice, dwIoControlCode, &s) ) < 0)
	{
 		printf("\n%s %s: ioctl cmd = 0x%lx failed with status = 0x%x\n",
			__FILE__,__FUNCTION__, dwIoControlCode, ioctl_status);
    	return FALSE;
 	}

	if (ioctl_status == IMB_SUCCESS)
	{
     	return TRUE;
	}
	else
	{
    	return FALSE;
    }
}
