/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2011 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/

#include "RemoteLogger.h"
#include "IpmiRequest.h"
#include "IpmiResponse.h"

#include "IpmiDCMIImpl.h"
#include "CExcept.h"
#include "DCMI_ioctl.h"

#define STR_VALUE(val1, val2) #val1 ": " #val2

#define CHECK_HANDLE(bool_exp, funcName) \
   if ( bool_exp ) { \
      throw CError( STR_VALUE(__FUNCTION__ , funcName) ); \
   }

IpmiDCMIImpl::IpmiDCMIImpl(void)
{
   LOG_DEBUG << "Opening DCMI Device";
#ifdef WIN32
	// Open the device
   std::string DrvName("\\\\.\\DCMI");
   h_ipmiDrv = CreateFile(DrvName.c_str(),  // Name of the "device" to open
									GENERIC_READ | GENERIC_WRITE,
									FILE_SHARE_READ | FILE_SHARE_WRITE,
									NULL,
									OPEN_EXISTING,
									FILE_ATTRIBUTE_NORMAL,
									NULL);

   CHECK_HANDLE( h_ipmiDrv==INVALID_HANDLE_VALUE, "CreateFile(\\\\.\\DCMI,..)" );
#else
   std::string DrvName("/dev/dcmi");
   h_ipmiDrv = open(DrvName.c_str(), O_RDWR);

   CHECK_HANDLE( h_ipmiDrv<0, "open(/dev/dcmi,..)" );
#endif
}

IpmiDCMIImpl::~IpmiDCMIImpl(void)
{
#ifdef WIN32
	if (h_ipmiDrv != INVALID_HANDLE_VALUE)
	{
		CloseHandle(h_ipmiDrv);
	}
#else
	if (h_ipmiDrv > -1)
	{
		close(h_ipmiDrv);
	}
#endif
}

IpmiResponse
IpmiDCMIImpl::requestResponse(const IpmiRequest& req)
{
   LOG_DEBUG << "Dcmi over Heci request-response invoked";

   BYTE requestData[1024]  = {0}; //what if total dcmi struct + red.data.size is bigger that this 1024 buffer

   ImbRequestBuffer  *reqBuff  = reinterpret_cast<ImbRequestBuffer *>(requestData);

   reqBuff->req.netFn = req.getNetFn();
   reqBuff->req.cmd   = req.getCmd();
   reqBuff->req.rsLun = req.getLun();
   reqBuff->req.rsSa  = 0x00;//Dont care
   reqBuff->req.dataLength = static_cast<BYTE>(req.getData().size());

#ifdef _MSC_VER
	std::copy(req.getData().begin(), req.getData().end(), stdext::checked_array_iterator<byte_t*>(reqBuff->req.data, sizeof(requestData)-sizeof(ImbRequestBuffer)));
#else
	std::copy(req.getData().begin(), req.getData().end(), reqBuff->req.data);
#endif

   reqBuff->timeOut = 10 * 1000;
   reqBuff->flags = 0;

   BYTE responseData[1024] = {0};

   ImbResponseBuffer *respBuff = reinterpret_cast<ImbResponseBuffer *>(responseData);
   DWORD respLength = sizeof( responseData );

   CHECK_HANDLE(!DeviceIoControl(h_ipmiDrv,
				(DWORD)IOCTL_DCMI_SEND_GET_MESSAGE,
				reqBuff,      // Ptr to InBuffer
				sizeof(requestData),
				respBuff,      // Ptr to OutBuffer
				sizeof(responseData),// Length of OutBuffer
				&respLength, // BytesReturned
				0), "DeviceIoControl" );

	IpmiResponse rsp(respBuff->cCode, respLength, &(respBuff->cCode));
	return rsp;
}
