/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/

#include <cstring>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <linux/ipmi.h>
#include <strings.h>

#include "IpmiProxy.h"
#include "IpmiDevIpmi0Impl.h"

#include "IpmiRequest.h"
#include "IpmiResponse.h"
#include "CExcept.h"

#include "DeviceIoControl.h"
#include "DCMI_ioctl.h"

using namespace std;

IpmiDevIpmi0Impl::IpmiDevIpmi0Impl()
{
    ipmiReqMsgId = 0;

	std::string DrvName("/dev/ipmi0");
    ipmiDrvHandle = open(DrvName.c_str(), O_RDWR);
    if (ipmiDrvHandle < 0)
    {
        throw CError("Cannot open IPMI device");
    }
    int events = 0;
    if (ioctl(ipmiDrvHandle, IPMICTL_SET_GETS_EVENTS_CMD, &events) < 0)
    {
    	throw CError("Cannot configure events");
    }

    unsigned int bmcSlaveAddr = 0x20;
    if (ioctl(ipmiDrvHandle, IPMICTL_SET_MY_ADDRESS_CMD, &bmcSlaveAddr) < 0)
    {
    	throw CError("Cannot open IPMI device");
    }
}

IpmiDevIpmi0Impl::~IpmiDevIpmi0Impl()
{
    if (ipmiDrvHandle >= 0)
    {
        close(ipmiDrvHandle);
    }
}

IpmiResponse
IpmiDevIpmi0Impl::requestResponse(const IpmiRequest& req)
{
    if (ipmiDrvHandle < 0)
    {
        throw CError("Driver handle error");
    }

    struct ipmi_system_interface_addr bmcAddr;
    memset(&bmcAddr, 0, sizeof(bmcAddr));
    bmcAddr.lun = req.getLun();
    bmcAddr.addr_type = IPMI_SYSTEM_INTERFACE_ADDR_TYPE;
    bmcAddr.channel = IPMI_BMC_CHANNEL;

    struct ipmi_req ipmiDrvRequest;
    memset(&ipmiDrvRequest, 0, sizeof(ipmiDrvRequest));
    ipmiDrvRequest.addr = (unsigned char*)&bmcAddr;
    ipmiDrvRequest.addr_len = sizeof(bmcAddr);
    ipmiDrvRequest.msgid = ipmiReqMsgId++;

    ipmiDrvRequest.msg.netfn = req.getNetFn();
    ipmiDrvRequest.msg.cmd = req.getCmd();

    std::vector<unsigned char> data = req.getData();
    ipmiDrvRequest.msg.data_len = data.size();
    ipmiDrvRequest.msg.data = new unsigned char[data.size()];
    for (unsigned int idx = 0; idx < data.size(); idx++)
    {
        ipmiDrvRequest.msg.data[idx] = data[idx];
    }

    if (ioctl(ipmiDrvHandle, IPMICTL_SEND_COMMAND, &ipmiDrvRequest) < 0)
    {
        throw CError("Cannot sendIPMI command");
    }
   // delete ipmiDrvRequest.msg.data; //ks wtf ??

    fd_set descriptors;
    FD_ZERO(&descriptors);
    FD_SET(ipmiDrvHandle, &descriptors);

    struct timeval waitTimeout;
    waitTimeout.tv_sec = 60;
    waitTimeout.tv_usec = 00;

    if (select(ipmiDrvHandle+1, &descriptors, NULL, NULL, &waitTimeout) < 0)
    {
        throw CError("Failed to wait for IPMI response");
    }

    if (!FD_ISSET(ipmiDrvHandle, &descriptors))
    {
        throw CError("Timeout waiting for IPMI response", 0);
    }

    struct ipmi_recv ipmiDrvResp;
    struct ipmi_addr ipmiDrvRespAddr;
    unsigned char respDataBuffer[1024]; //TBD: Replace 1024 with some constant.

    ipmiDrvResp.addr = (unsigned char *) &ipmiDrvRespAddr;
    ipmiDrvResp.addr_len = sizeof(ipmiDrvRespAddr);
    ipmiDrvResp.msg.data = respDataBuffer;
    ipmiDrvResp.msg.data_len = sizeof(respDataBuffer);

    if (ioctl(ipmiDrvHandle, IPMICTL_RECEIVE_MSG_TRUNC, &ipmiDrvResp) < 0)
    {
        throw CError("Error reading IPMI response");
    }

//Anandhi     KcsIpmiResponse resp(ipmiDrvResp.msg.data[0], ipmiDrvResp.msg.data_len-1, respDataBuffer);
    IpmiResponse resp(ipmiDrvResp.msg.data[0], ipmiDrvResp.msg.data_len, respDataBuffer);
    return resp;
}

