/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
#ifndef _IPMI_REQUEST_H
#define _IPMI_REQUEST_H

#include <vector>
#include <iostream>

class IpmiRequest
{
public:
    /**
     * Ctor.
     *
     * @param dest Destination - BMC address (currently not used due to
     *    compatibility with existing Linux server implementation).
     * @param netFn Network function.
     * @param lun Lun.
     * @param cmd Command.
     * @param dataLen Length of the <code>data</code> array.
     * @param data Data.
     */
    IpmiRequest(unsigned char dest, unsigned char netFn, unsigned char lun,
                         unsigned int cmd, unsigned int dataLen, const unsigned char* data);

    /**
     * Retrieves the Destination.
     *
     * @return Destination.
     */
    inline unsigned char getDest() const;

    /**
     * Retrieves the Network Function.
     *
     * @return Network Function.
     */
    inline unsigned char getNetFn() const;

    /**
     * Retrieves the Lun.
     *
     * @return Lun.
     */
    inline unsigned char getLun() const;

    /**
     * Retrieves teh Command.
     *
     * @return Command.
     */
    inline unsigned char getCmd() const;

    /**
     * Retrieves the Data.
     *
     * @return Data.
     */
    inline const std::vector<unsigned char>& getData() const;

private:
    /// Destination.
    unsigned char dest;

    /// Network Function.
    unsigned char netFn;

    /// Lun.
    unsigned char lun;

    /// Command.
    unsigned char cmd;

    /// Data.
    std::vector<unsigned char> data;
};

inline
unsigned char
IpmiRequest::getDest() const
{
    return dest;
}

inline
unsigned char
IpmiRequest::getNetFn() const
{
    return netFn;
}

inline
unsigned char
IpmiRequest::getLun() const
{
    return lun;
}

inline
unsigned char
IpmiRequest::getCmd() const
{
    return cmd;
}

inline
const std::vector<unsigned char>&
IpmiRequest::getData() const
{
    return data;
}

/**
 * Dumps content of the request to the specified output stream.
 *
 * @param os Destination output stream.
 * @param req Request.
 *
 * @return Reference to the <code>os</code>.
 */
std::ostream& operator<<(std::ostream& os, const IpmiRequest& req);

#endif //#ifndef _IPMI_REQUEST_H
