/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/

#ifndef _INBAND_IPMI_RESPONSE
#define _INBAND_IPMI_RESPONSE

#include <vector>
#include "OsSpecificSockets.h"

class IpmiResponse
{
public:
    /**
     * Ctor.
     *
     * @param completionCode Completion Code.
     * @param dataLen Specifies length of the <code>data</code>.
     * @param data Data.
     */
    IpmiResponse(unsigned char completionCode, unsigned int dataLen, unsigned char* data);

    /**
     * Retrieves the completion code.
     *
     * @return Completion Code.
     */
    inline unsigned char getCompletionCode() const;

    /**
     * Retrieves the data.
     *
     * @return Data.
     */
    inline const std::vector<unsigned char>& getData() const;

private:
    /**
     * Default ctor for internal use.
     */
    IpmiResponse();

    // Completion Code.
    unsigned char completionCode;

    // Data.
    std::vector<unsigned char> data;
};

inline
unsigned char
IpmiResponse::getCompletionCode() const
{
    return completionCode;
}

inline
const std::vector<unsigned char>&
IpmiResponse::getData() const
{
    return data;
}

/**
 * Dumps content of the request to the specified output stream.
 *
 * @param os Destination output stream.
 * @param req Request.
 *
 * @return Reference to the <code>os</code>.
 */
std::ostream& operator<<(std::ostream& os, const IpmiResponse& resp);

#endif //#ifndef _INBAND_IPMI_RESPONSE
