/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/

#ifndef _IPMI_WMIIMPL_H
#define _IPMI_WMIIMPL_H

#include "ipmiproxy.h"

#include <atlbase.h>
#include <wbemidl.h>
#include <comdef.h>
#include <atlsafe.h>


/**
 *  IPMI over Microsoft WMI or linux ... implementation ...
 */

class IpmiWMIImpl :
   public IpmiProxy
{
public:
    /**
     * The default ctor opens a connection to the local
     * Wbem service and obtains a path to the Microsoft_IPMI
     * object. If Wbem cannot be accessed or Microsoft_IPMI
     * is not accessible, an exception is thrown.
     */
    IpmiWMIImpl();

    /**
     * Dtor.
     */
    virtual ~IpmiWMIImpl();

    /**
     * Invokes Microsoft_IPMI::RequestResponse method.
     *
     * @param req IPMI request that is passed to the Microsoft_IPMI.
     * @return IPMI response that is retrieved from the Microsoft_IPMI.
     */
    IpmiResponse requestResponse(const IpmiRequest& req);

private:
    /// Ptr to the IWbemServices interface.
    CComPtr<IWbemServices> wbemServices;

    /// Path the Microsoft_IPMI instance.
    bstr_t ipmiInstancePath;

    /// BMC Address retrieved from the Microsoft_IPMI instance.
    unsigned char bmcAddress;

    /// Signature of [in] parameters to the Microsoft_IPMI::RequestResponse.
    CComPtr<IWbemClassObject> inParamsDef;
};

#endif //#ifndef _IPMI_WMIIMPL_H
