/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008,2011 INTEL CORPORATION                  **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
#include <iostream>

#include "TransportPluginImpl.h"
#include "IpmiRequest.h"
#include "IpmiResponse.h"

#include "transport.h"

using namespace std;
using namespace InBandTransportPlugin;

#ifdef WIN32

BOOL
APIENTRY
DllMain(HMODULE hModule,
        DWORD ul_reason_for_call,
        LPVOID lpReserved
)
{
    switch (ul_reason_for_call)
    {
    case DLL_PROCESS_ATTACH:
        // do the necesasry initialization
        try
        {
            TransportPluginImpl::initialize();
        }
        catch (exception& e)
        {
            cerr << "ERROR: Failed to initialize TCP Inband Transport Plug-in." << endl
            << e.what() << endl;
            return FALSE;
        }
        catch (...)
        {
            cerr << "ERROR: Failed to initialize TCP Inband Transport Plug-in. Unknown error." << endl;
            return FALSE;
        }
        DisableThreadLibraryCalls(hModule);
        break;

    case DLL_PROCESS_DETACH:
        TransportPluginImpl::destroy();
        break;

    default:
        break;
    }
    return TRUE;
}

#endif // WIN32

EXPORTS
int
Dcmi_Initialize(const dcmi_init_info_t* initInfo)
{
    return TransportPluginImpl::getInstance()->start(initInfo);
}

EXPORTS
int
Dcmi_Shutdown()
{
    return TransportPluginImpl::getInstance()->shutdown();
}

EXPORTS
int
Dcmi_Configure(const char* paramName, const char* val)
{
    return TransportPluginImpl::getInstance()->configure(paramName, val);
}

EXPORTS
int
Dcmi_Connect(void)
{
    return TransportPluginImpl::getInstance()->connect();
}


EXPORTS
int
Dcmi_OpenSession(void)
{
	return E_OK;
}

EXPORTS
int
Dcmi_CloseSession(void)
{
	return E_OK;
}

EXPORTS
int
Dcmi_SendCmd(const struct dcmi_req_t* req, struct dcmi_rsp_t* rsp)
{
    return TransportPluginImpl::getInstance()->sendIpmiCmd(req, rsp);
}

EXPORTS
int
Dcmi_Disconnect(void)
{
    return TransportPluginImpl::getInstance()->disconnect();
}
