/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/

#include <stdexcept>
#include <iostream>

#include "transport.h"

#include "RmcppTransportPluginImpl.h"

using namespace std;
using namespace RmcppTransportPlugin;

#ifdef WIN32

#include <windows.h>

BOOL
APIENTRY
DllMain(HMODULE module, DWORD reasonForCall, LPVOID reserved)
{
    UNREFERENCED_PARAMETER(reserved);

    switch (reasonForCall)
    {
    case DLL_PROCESS_ATTACH:
        // do the necessary initialization
        try
        {
            RmcppTransportPluginImpl::initialize();
        }
        catch (exception& e)
        {
            cerr << "ERROR: Failed to initialize RMCPP Transport Plug-in." << endl
            << e.what() << endl;
            return FALSE;
        }
        catch (...)
        {
            cerr << "ERROR: Failed to initialize RMCPP Transport Plug-in. Unknown error." << endl;
            return FALSE;
        }
        DisableThreadLibraryCalls(module);
        break;

    case DLL_PROCESS_DETACH:
        RmcppTransportPluginImpl::destroy();
        break;

    default:
        break;
    }
    return TRUE;
}

#endif //WIN32

EXPORTS
int
Dcmi_Initialize(const dcmi_init_info_t* initInfo)
{
    return RmcppTransportPluginImpl::getInstance()->start(initInfo);
}

EXPORTS
int
Dcmi_Shutdown()
{
    return RmcppTransportPluginImpl::getInstance()->shutdown();
}

EXPORTS
int
Dcmi_Configure(const char* paramName, const char* val)
{
    return RmcppTransportPluginImpl::getInstance()->configure(paramName, val);
}

EXPORTS
int
Dcmi_Connect(void)
{
    return RmcppTransportPluginImpl::getInstance()->connect();
}


EXPORTS
int
Dcmi_OpenSession(void)
{
	return RmcppTransportPluginImpl::getInstance()->open_session();
}

EXPORTS
int
Dcmi_CloseSession(void)
{
	return RmcppTransportPluginImpl::getInstance()->close_session();
}


EXPORTS
int
Dcmi_SendCmd(const struct dcmi_req_t* req, struct dcmi_rsp_t* rsp)
{
    return RmcppTransportPluginImpl::getInstance()->sendIpmiCmd(req, rsp);
}

EXPORTS
int
Dcmi_Disconnect(void)
{
    return RmcppTransportPluginImpl::getInstance()->disconnect();
}

#ifdef SOL
EXPORTS
int
Dcmi_RecvData(const timeval timeoutVal, bool *timeout, dcmi_rsp_t* rsp)
{
	return RmcppTransportPluginImpl::getInstance()->recvSessionPayload(timeoutVal, timeout, rsp);
}

EXPORTS
int
Dcmi_SendData(const dcmi_req_t* req)
{
    return RmcppTransportPluginImpl::getInstance()->sendSessionPayload(req);
}
#endif

