/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/
/*******************************************************************************
***		 File		:	TransportPluginDll.h								  **
***																			  **
********************************************************************************/

#ifndef _TRANSPORT_PLUGIN_DLL_H
#define _TRANSPORT_PLUGIN_DLL_H

#include <memory>

// forward declarations
struct dcmi_req_t;
struct dcmi_rsp_t;
struct dcmi_init_info_t;

/// Type of  pointer to the <code>Dcmi_Initialize()</code> of Transport Plug-in API.
typedef int (*dcmi_initialize_t)(const dcmi_init_info_t* initInfo);

/// Type of  pointer to the <code>Dcmi_Shutdown()</code> of Transport Plug-in API.
typedef int (*dcmi_shutdown_t)();

/// Type of  pointer to the <code>Dcmi_Configure()</code> of Transport Plug-in API.
typedef int (*dcmi_configure_t)(const char* param_name, const char* val);

/// Type of pointer to the <code>Dcmi_Connect()</code> of Transport Plug-in API.
typedef int (*dcmi_connect_t)();

/// Type of pointer to the <code>Dcmi_OpenSession()</code> of Transport Plug-in API.
typedef int (*dcmi_open_session_t)();

/// Type of pointer to the <code>Dcmi_ClosseSession()</code> of Transport Plug-in API.
typedef int (*dcmi_close_session_t)();

/// Type of pointer to the <code>Dcmi_SendCmd()</code> of Transport Plug-in API.
typedef int (*dcmi_send_ipmi_cmd_t)(const struct dcmi_req_t* req, struct dcmi_rsp_t* rsp);

/// Type of pointer to the <code>Dcmi_Disconnect()</code> of Transport Plug-in API.
typedef int (*dcmi_disconnect_t)();

/**
 * Defines interface of a Transport Plug-in library.
 */
struct transport_plugin_interface
{
    /// Entrypoint of the <code>Dcmi_Initialize()</code>.
    dcmi_initialize_t initialize;

    /// Entrypoint of the <code>Dcmi_Shutdown()</code>.
    dcmi_shutdown_t shutdown;

    /// Entrypoint of the <code>Dcmi_Configure()</code>.
    dcmi_configure_t configure;

    /// Entrypoint of the <code>Dcmi_Connect()</code>.
    dcmi_connect_t connect;

    /// Entrypoint of the <code>Dcmi_SendCmd()</code>.
    dcmi_send_ipmi_cmd_t sendIpmiCmd;

    /// Entrypoint of the <code>dcmi_diconnect()</code>.
    dcmi_disconnect_t disconnect;

    /// Entrypoint of the <code>Dcmi_OpenSession()</code>.
    dcmi_open_session_t open_session;

      /// Entrypoint of the <code>Dcmi_ClosseSession()</code>.
    dcmi_close_session_t close_session;
};

/**
 * Numeric plugin ID and its name (unique as well).
 */
struct PluginId
{
    /// Numeric plugin ID.
    unsigned int id;
    /// Plugin name.
    const char* name;
};

/**
 * Transport Plugin DLL hides OS-specific implementation of loading,
 * unloading and obtaining interface to dynamically linked libraries.
 */
class TransportPluginDll
{
public:
    /**
     * Constructor: attempts to load the dynamic library.
     *
     * @param pluginName Name of the DLL plugin.
     */
    TransportPluginDll(const char* pluginName);

    /**
     * Destructor: unloads the dynamic library.
     */
    ~TransportPluginDll();

    /**
     * Obtains dynamic library interface
     *
     * @param pTpi pointer to the interface struct.
     *
     * @returns NONE.
     */
    void GetInterface(transport_plugin_interface* pTpi);

protected:

    /// Handle to the DLL.
    //struct transport_dll_handle *pDllHandle;
	std::auto_ptr<struct transport_dll_handle> DllHandle;

    /**
     * Retrieves a pointer to a function of type <code>T</code> from
     * the dynamic library.
     *
     * @param procName Name of the function to retrieve the entrypoint to.
     *
     * @return Pointer to the function.
     */
    template<class T> T getProcAddress(const char* procName);
};

#endif //#ifndef _TRANSPORT_PLUGIN_DLL_H
