/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008, 2011 INTEL CORPORATION                 **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intels	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/

#include <sstream>
#include <stdexcept>
#include <ltdl.h>
#include <cstdlib>
#include "TransportPluginDll.h"
#include "version.h"

#ifndef MODULE_PATH_ENV
#  define MODULE_PATH_ENV        "MODULE_PATH"
#endif

using namespace std;

struct transport_dll_handle
{
    lt_dlhandle handle;
};

TransportPluginDll::~TransportPluginDll()
{
    if (NULL != DllHandle->handle)
    {
        lt_dlclose(DllHandle->handle);
    }
//    delete pDllHandle;
    lt_dlexit();
}

template<class T>
T
TransportPluginDll::getProcAddress(const char* procName)
{
    if (NULL == DllHandle->handle)
    {
        throw logic_error("NULL pointer to library");
    }
    T addr = (T)(lt_dlsym(DllHandle->handle, procName));
    if (NULL == addr)
    {
        throw runtime_error(string("Failed get address of ") + procName);
    }
    return addr;
}

void
TransportPluginDll::GetInterface(transport_plugin_interface* pTpi)
{
    pTpi->initialize = getProcAddress<dcmi_initialize_t>("Dcmi_Initialize");
    pTpi->shutdown = getProcAddress<dcmi_shutdown_t>("Dcmi_Shutdown");
    pTpi->configure = getProcAddress<dcmi_configure_t>("Dcmi_Configure");
    pTpi->connect = getProcAddress<dcmi_connect_t>("Dcmi_Connect");
    pTpi->sendIpmiCmd = getProcAddress<dcmi_send_ipmi_cmd_t>("Dcmi_SendCmd");
    pTpi->disconnect = getProcAddress<dcmi_disconnect_t>("Dcmi_Disconnect");
    pTpi->open_session = getProcAddress<dcmi_open_session_t>("Dcmi_OpenSession");
    pTpi->close_session = getProcAddress<dcmi_close_session_t>("Dcmi_CloseSession");
}

TransportPluginDll::TransportPluginDll(const char* pluginName)
{
    if (lt_dlinit())
    {
        stringstream msg;
        msg << "Plugin: " << pluginName << ". Failed to initialize libltdl. Error msg: " << lt_dlerror();
        throw runtime_error(msg.str());
    }

    // Attempt to load the DLL
    DllHandle = std::auto_ptr<struct transport_dll_handle>(new struct transport_dll_handle);
    stringstream libraryName;
    libraryName << "lib" << pluginName << ".la";

    const char *modPath = getenv(MODULE_PATH_ENV);
    if (modPath != NULL)
    {
        if (!lt_dlsetsearchpath(modPath))
        {
            stringstream msg;
            msg << "Plugin: " << pluginName << ". Failed lt_dlsetsearchpath. Error msg: " << lt_dlerror();
            throw runtime_error(msg.str());
        }
    }

    DllHandle->handle = lt_dlopenext(libraryName.str().c_str());

    if (!DllHandle->handle)
    {
        lt_dlclose(DllHandle->handle);

        stringstream msg;
        msg << "Failed to load plug-in dll " << pluginName << ": LoadLibrary() Error " << lt_dlerror();
        throw runtime_error(msg.str());
    }
}

