/*******************************************************************************
********************************************************************************
***                                                                           **
***							INTEL CONFIDENTIAL								  **
***                    COPYRIGHT 2008 INTEL CORPORATION                       **
***							All Rights Reserved								  **
***                                                                           **
***                INTEL CORPORATION PROPRIETARY INFORMATION                  **
***                                                                           **
***		The source code contained or described herein and all documents		  **
***		related to the source code ("Material") are owned by Intel			  **
***		Corporation or its suppliers or licensors. Title to the Material	  **
***		remains with Intel Corporation or its suppliers and licensors.		  **
***		The Material contains trade secrets and proprietary and confidential  **
***		information of Intel or its suppliers and licensors.				  **
***		The Material is protected by worldwide copyright and trade secret	  **
***		laws and treaty provisions. No part of the Material may be used,	  **
***		copied, reproduced, modified, published, uploaded, posted,			  **
***		transmitted, distributed, or disclosed in any way without Intel�s	  **
***		prior express written permission.									  **
***																			  **
***		No license under any patent, copyright, trade secret or other		  **
***		intellectual property right is granted to or conferred upon you by	  **
***		disclosure or delivery of the Materials, either expressly, by		  **
***		implication, inducement, estoppel or otherwise. Any license under	  **
***		such intellectual property rights must be express and approved by	  **
***		Intel in writing.													  **
********************************************************************************/

#include <sstream>
#include <stdexcept>

#include "TransportPluginLoader.h"
#include "Logger.h"
#include "transport.h"

using namespace std;

TransportPluginLoader::~TransportPluginLoader()
{
    for (size_t i = 0; i < plugins.size(); ++i)
    {
        delete plugins[i];
    }
}

TransportPluginLoader::TransportPlugin::~TransportPlugin()
{
    tpi.shutdown();
    delete this->pDll;
}

TransportPluginLoader::TransportPlugin::TransportPlugin(unsigned int pluginId,
                                                        const char* pluginName)
{
    this->name = pluginName;
    this->id = pluginId;

    try
    {
        this->pDll = new TransportPluginDll(pluginName);
        pDll->GetInterface(&this->tpi);

        dcmi_init_info_t pluginInitInfo;
        pluginInitInfo.loggerFun = ::TransportPlugin::Logger::log;
        tpi.initialize(&pluginInitInfo);
    } catch (exception& e)
    {
        throw runtime_error(string(pluginName) + " " + e.what());
    }
}

void
TransportPluginLoader::load(const std::vector<PluginId>& pluginIds)
{
    // do not try to load more than once
    if (!plugins.empty())
    {
        throw logic_error("Already loaded");
    }

    plugins.reserve(pluginIds.size());
    for (size_t i = 0; i < pluginIds.size(); ++i)
    {
        TransportPlugin* pPlugin = new TransportPlugin(pluginIds[i].id, pluginIds[i].name);

        // Add to the internal collections
        plugins.push_back(pPlugin);
        id2plugin[pPlugin->id] = plugins[plugins.size()-1];
        name2plugin[pPlugin->name] = plugins[plugins.size()-1];
    }
}

transport_plugin_interface*
TransportPluginLoader::getPluginById(unsigned int id) const
{
    map<unsigned int, TransportPlugin*>::const_iterator plugin = id2plugin.find(id);
    if (plugin == id2plugin.end())
    {
        throw runtime_error("Plugin not found");
    }
    return &plugin->second->tpi;
}

transport_plugin_interface*
TransportPluginLoader::getPluginByName(const char* name) const
{
    map<string, TransportPlugin*>::const_iterator plugin = name2plugin.find(name);
    if (plugin == name2plugin.end())
    {
        throw runtime_error("Plugin not found");
    }
    return &plugin->second->tpi;
}
//*************************************************************************
